#!/usr/bin/env python2.4
"""
Returns all positions of a maf with any pwm score > threshold
The positions are projected onto human coordinates
"""
from __future__ import print_function

import sys

import bx.pwm.position_weight_matrix as pwmx
from bx.align import maf as align_maf
from bx.pwm.pwm_score_maf import MafBlockScorer


def isnan(x):
    return not x == x


def main():

    if len(sys.argv) < 6:
        print("%s transfac|basic pwmfile inmaf threshold spec1,spec2,... " % sys.argv[0], file=sys.stderr)
        sys.exit(0)

    pwm = {}
    format = sys.argv[1]
    for wm in pwmx.Reader(open(sys.argv[2]), format=format):
        pwm[wm.id] = wm

    inmaf = open(sys.argv[3])
    threshold = float(sys.argv[4])

    species = []

    for sp in sys.argv[5].split(','):
        species.append(sp)

    for maf in align_maf.Reader(inmaf):
        mafchrom = maf.components[0].src.split('.')[1]
        mafstart = maf.components[0].start
        mafend = maf.components[0].end
        reftext = maf.components[0].text

        # maf block scores for each matrix
        for scoremax, width, headers in MafBlockScorer(pwm, species, maf):
            blocklength = width
            mafsrc, mafstart, mafend = headers[0]
            mafchrom = mafsrc.split('.')[1]

            # lists of scores for each position in scoremax
            for id, mx in scoremax.items():
                for offset in range(blocklength):

                    # scan all species with threshold
                    for i in range(len(species)):
                        if mx[i][offset] > threshold:
                            refstart = mafstart + offset - reftext.count('-', 0, offset)
                            refend = refstart + len(pwm[id])
                            data = " ".join(["%.2f" % mx[x][offset] for x in range(len(species))])
                            # underscore spaces in the name
                            print(mafchrom, refstart, refend, id.replace(' ', '_'), data)
                            break


if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
Read a table dump in the UCSC gene table format and print a tab separated
list of intervals corresponding to requested features of each gene.

usage: ucsc_gene_table_to_intervals.py [options] < gene_table.txt

options:
  -h, --help            show this help message and exit
  -rREGION, --region=REGION
                        Limit to region: one of coding, utr3, utr5, transcribed [default]
  -e, --exons           Only print intervals overlapping an exon
"""
from __future__ import print_function

import optparse
import string
import sys


def main():
    # Parse command line
    parser = optparse.OptionParser(usage="%prog [options] < gene_table.txt")
    parser.add_option("-r", "--region", dest="region", default="transcribed",
                      help="Limit to region: one of coding, utr3, utr5, transcribed [default]")
    parser.add_option("-e", "--exons", action="store_true", dest="exons",
                      help="Only print intervals overlapping an exon")
    parser.add_option("-s", "--strand", action="store_true", dest="strand",
                      help="Print strand after interval")
    parser.add_option("-b", "--nobin", action="store_false", dest="discard_first_column", default=True,
                      help="file doesn't contain a 'bin' column (use this for pre-hg18 files)")
    options, args = parser.parse_args()
    assert options.region in ('coding', 'utr3', 'utr5', 'transcribed'), "Invalid region argument"

    # Read table from stdin and handle each gene
    for line in sys.stdin:

        # Parse fields from gene tabls
        fields = line.split('\t')
        if options.discard_first_column:
            fields.pop(0)
        chrom = fields[1]
        strand = fields[2]
        tx_start = int(fields[3])
        tx_end = int(fields[4])
        cds_start = int(fields[5])
        cds_end = int(fields[6])

        # Determine the subset of the transcribed region we are interested in
        if options.region == 'utr3':
            if strand == '-':
                region_start, region_end = tx_start, cds_start
            else:
                region_start, region_end = cds_end, tx_end
        elif options.region == 'utr5':
            if strand == '-':
                region_start, region_end = cds_end, tx_end
            else:
                region_start, region_end = tx_start, cds_start
        elif options.region == 'coding':
            region_start, region_end = cds_start, cds_end
        else:
            region_start, region_end = tx_start, tx_end

        # If only interested in exons, print the portion of each exon overlapping
        # the region of interest, otherwise print the span of the region
        if options.exons:
            exon_starts = [int(_) for _ in fields[8].rstrip(',\n').split(',')]
            exon_ends = [int(_) for _ in fields[9].rstrip(',\n').split(',')]
            for start, end in zip(exon_starts, exon_ends):
                start = max(start, region_start)
                end = min(end, region_end)
                if start < end:
                    if strand:
                        print_tab_sep(chrom, start, end, strand)
                    else:
                        print_tab_sep(chrom, start, end)
        else:
            if strand:
                print_tab_sep(chrom, region_start, region_end, strand)
            else:
                print_tab_sep(chrom, region_start, region_end)


def print_tab_sep(*args):
    """Print items in `l` to stdout separated by tabs"""
    print(string.join((str(f) for f in args), '\t'))


if __name__ == "__main__":
    main()



"""
Support for scoring alignments using arbitrary scoring matrices, arbitrary
alphabets, and affine gap penalties.
"""

from numpy import (
    float32,
    int32,
    ones,
    zeros
)


class ScoringScheme(object):
    # note that gap_open and gap_extend are penalties, which means you should make them positive
    def __init__(self, gap_open, gap_extend, default=-100, alphabet1="ACGT", alphabet2=None, gap1="-", gap2=None, text1_range=128, text2_range=None, typecode=int32):
        if text2_range is None:
            text2_range = text1_range
        if alphabet2 is None:
            alphabet2 = alphabet1
        if gap2 is None:
            gap2 = gap1  # (scheme with gap1=gap2=None is legit)
        if isinstance(alphabet1, str):
            alphabet1 = [ch for ch in alphabet1]
        if isinstance(alphabet2, str):
            alphabet2 = [ch for ch in alphabet2]
        self.table = ones((text1_range, text2_range), typecode)
        self.table *= default
        self.gap_open = gap_open
        self.gap_extend = gap_extend
        self.gap1 = gap1
        self.gap2 = gap2
        self.alphabet1 = alphabet1
        self.alphabet2 = alphabet2
        # private _set_score and _get_score allow subclasses to override them to
        # implement a different underlying table object

    def _set_score(self, a_b_pair, val):
        (a, b) = a_b_pair
        self.table[a, b] = val

    def _get_score(self, a_b_pair):
        (a, b) = a_b_pair
        return self.table[a, b]

    def set_score(self, a, b, val, foldcase1=False, foldcase2=False):
        self._set_score((a, b), val)
        if foldcase1:
            aCh = chr(a)
            if (aCh.isupper()):
                aa = ord(aCh.lower())
            elif (aCh.islower()):
                aa = ord(aCh.upper())
            else:
                foldcase1 = False
        if foldcase2:
            bCh = chr(b)
            if (bCh.isupper()):
                bb = ord(bCh.lower())
            elif (bCh.islower()):
                bb = ord(bCh.upper())
            else:
                foldcase2 = False
        if foldcase1 and foldcase2:
            self._set_score((aa, b), val)
            self._set_score((a, bb), val)
            self._set_score((aa, bb), val)
        elif foldcase1:
            self._set_score((aa, b), val)
        elif foldcase2:
            self._set_score((a, bb), val)

    def score_alignment(self, a):
        return score_alignment(self, a)

    def score_texts(self, text1, text2):
        return score_texts(self, text1, text2)

    def __str__(self):
        isDna1 = "".join(self.alphabet1) == "ACGT"
        isDna2 = "".join(self.alphabet2) == "ACGT"
        labelRows = not (isDna1 and isDna2)
        width = 3
        for a in self.alphabet1:
            for b in self.alphabet2:
                score = self._get_score((ord(a), ord(b)))
                if (isinstance(score, float)):
                    s = "%8.6f" % score
                else:
                    s = "%s" % score
                if (len(s)+1 > width):
                    width = len(s)+1
        lines = []
        line = []
        if labelRows:
            if isDna1:
                line.append(" ")
            else:
                line.append("  ")
        for b in self.alphabet2:
            if isDna2:
                s = b
            else:
                s = "%02X" % ord(b)
            line.append("%*s" % (width, s))
        lines.append(("".join(line))+"\n")
        for a in self.alphabet1:
            line = []
            if labelRows:
                if isDna1:
                    line.append(a)
                else:
                    line.append("%02X" % ord(a))
            for b in self.alphabet2:
                score = self._get_score((ord(a), ord(b)))
                if (isinstance(score, float)):
                    s = "%8.6f" % score
                else:
                    s = "%s" % score
                line.append("%*s" % (width, s))
            lines.append(("".join(line))+"\n")
        return "".join(lines)


def read_scoring_scheme(f, gap_open, gap_extend, gap1="-", gap2=None, **kwargs):
    """
    Initialize scoring scheme from a file containint a blastz style text blob.
    f can be either a file or the name of a file.
    """
    close_it = False
    if (isinstance(f, str)):
        f = open(f, "rt")
        close_it = True
    ss = build_scoring_scheme("".join([line for line in f]), gap_open, gap_extend, gap1=gap1, gap2=gap2, **kwargs)
    if (close_it):
        f.close()
    return ss


def build_scoring_scheme(s, gap_open, gap_extend, gap1="-", gap2=None, **kwargs):
    """
    Initialize scoring scheme from a blastz style text blob, first line
    specifies the bases for each row/col, subsequent lines contain the
    corresponding scores.  Slaw extensions allow for unusual and/or
    asymmetric alphabets.  Symbols can be two digit hex, and each row
    begins with symbol.  Note that a row corresponds to a symbol in text1
    and a column to a symbol in text2.

    examples:

       blastz                       slaw

          A    C    G    T               01   02    A    C    G    T
         91 -114  -31 -123          01  200 -200  -50  100  -50  100
       -114  100 -125  -31          02 -200  200  100  -50  100  -50
        -31 -125  100 -114
       -123  -31 -114   91
    """
    # perform initial parse to determine alphabets and locate scores
    bad_matrix = "invalid scoring matrix"
    s = s.rstrip("\n")
    lines = s.split("\n")
    rows = []
    symbols2 = lines.pop(0).split()
    symbols1 = None
    rows_have_syms = False
    a_la_blastz = True
    for i, line in enumerate(lines):
        row_scores = line.split()
        if len(row_scores) == len(symbols2):        # blastz-style row
            if symbols1 is None:
                if len(lines) != len(symbols2):
                    raise bad_matrix
                symbols1 = symbols2
            elif (rows_have_syms):
                raise bad_matrix
        elif len(row_scores) == len(symbols2) + 1:  # row starts with symbol
            if symbols1 is None:
                symbols1 = []
                rows_have_syms = True
                a_la_blastz = False
            elif not rows_have_syms:
                raise bad_matrix
            symbols1.append(row_scores.pop(0))
        else:
            raise bad_matrix
        rows.append(row_scores)
    # convert alphabets from strings to characters
    try:
        alphabet1 = [sym_to_char(sym) for sym in symbols1]
        alphabet2 = [sym_to_char(sym) for sym in symbols2]
    except ValueError:
        raise bad_matrix
    if (alphabet1 != symbols1) or (alphabet2 != symbols2):
        a_la_blastz = False
    if a_la_blastz:
        alphabet1 = [ch.upper() for ch in alphabet1]
        alphabet2 = [ch.upper() for ch in alphabet2]
    # decide if rows and/or columns should reflect case
    if a_la_blastz:
        foldcase1 = foldcase2 = True
    else:
        foldcase1 = "".join(alphabet1) == "ACGT"
        foldcase2 = "".join(alphabet2) == "ACGT"
    # create appropriately sized matrix
    text1_range = text2_range = 128
    if ord(max(alphabet1)) >= 128:
        text1_range = 256
    if ord(max(alphabet2)) >= 128:
        text2_range = 256
    typecode = int32
    for i, row_scores in enumerate(rows):
        for j, score in enumerate(map(int_or_float, row_scores)):
            if isinstance(score, float):
                typecode = float32
    if isinstance(gap_open, float):
        typecode = float32
    if isinstance(gap_extend, float):
        typecode = float32
    ss = ScoringScheme(gap_open, gap_extend, alphabet1=alphabet1, alphabet2=alphabet2, gap1=gap1, gap2=gap2, text1_range=text1_range, text2_range=text2_range, typecode=typecode, **kwargs)
    # fill matrix
    for i, row_scores in enumerate(rows):
        for j, score in enumerate(map(int_or_float, row_scores)):
            ss.set_score(ord(alphabet1[i]), ord(alphabet2[j]), score)
            if foldcase1 and foldcase2:
                ss.set_score(ord(alphabet1[i].lower()), ord(alphabet2[j].upper()), score)
                ss.set_score(ord(alphabet1[i].upper()), ord(alphabet2[j].lower()), score)
                ss.set_score(ord(alphabet1[i].lower()), ord(alphabet2[j].lower()), score)
            elif foldcase1:
                ss.set_score(ord(alphabet1[i].lower()), ord(alphabet2[j]), score)
            elif foldcase2:
                ss.set_score(ord(alphabet1[i]), ord(alphabet2[j].lower()), score)
    return ss


def int_or_float(s):
    try:
        return int(s)
    except ValueError:
        return float(s)

# convert possible two-char symbol to a single character


def sym_to_char(sym):
    if len(sym) == 1:
        return sym
    elif len(sym) != 2:
        raise ValueError
    else:
        return chr(int(sym, base=16))


def score_alignment(scoring_scheme, a):
    score = 0
    ncomps = len(a.components)
    for i in range(ncomps):
        for j in range(i+1, ncomps):
            score += score_texts(scoring_scheme, a.components[i].text, a.components[j].text)
    return score


def score_texts(scoring_scheme, text1, text2):
    rval = 0
    last_gap_a = last_gap_b = False
    for i in range(len(text1)):
        a = text1[i]
        b = text2[i]
        # Ignore gap/gap pair
        if a == scoring_scheme.gap1 and b == scoring_scheme.gap2:
            continue
        # Gap in first species
        elif a == scoring_scheme.gap1:
            rval -= scoring_scheme.gap_extend
            if not last_gap_a:
                rval -= scoring_scheme.gap_open
                last_gap_a = True
                last_gap_b = False
        # Gap in second species
        elif b == scoring_scheme.gap2:
            rval -= scoring_scheme.gap_extend
            if not last_gap_b:
                rval -= scoring_scheme.gap_open
                last_gap_a = False
                last_gap_b = True
        # Aligned base
        else:
            rval += scoring_scheme._get_score((ord(a), ord(b)))
            last_gap_a = last_gap_b = False
    return rval


def accumulate_scores(scoring_scheme, text1, text2, skip_ref_gaps=False):
    """
    Return cumulative scores for each position in alignment as a 1d array.

    If `skip_ref_gaps` is False positions in returned array correspond to each
    column in alignment, if True they correspond to each non-gap position (each
    base) in text1.
    """
    if skip_ref_gaps:
        rval = zeros(len(text1) - text1.count(scoring_scheme.gap1))
    else:
        rval = zeros(len(text1))
    score = 0
    pos = 0
    last_gap_a = last_gap_b = False
    for i in range(len(text1)):
        a = text1[i]
        b = text2[i]
        # Ignore gap/gap pair
        if a == scoring_scheme.gap1 and b == scoring_scheme.gap2:
            continue
        # Gap in first species
        elif a == scoring_scheme.gap1:
            score -= scoring_scheme.gap_extend
            if not last_gap_a:
                score -= scoring_scheme.gap_open
                last_gap_a = True
                last_gap_b = False
        # Gap in second species
        elif b == scoring_scheme.gap2:
            score -= scoring_scheme.gap_extend
            if not last_gap_b:
                score -= scoring_scheme.gap_open
                last_gap_a = False
                last_gap_b = True
        # Aligned base
        else:
            score += scoring_scheme._get_score((ord(a), ord(b)))
            last_gap_a = last_gap_b = False
        if not(skip_ref_gaps) or a != scoring_scheme.gap1:
            rval[pos] = score
            pos += 1
    return rval


hox70 = build_scoring_scheme("""  A    C    G    T
                                  91 -114  -31 -123
                                -114  100 -125  -31
                                 -31 -125  100 -114
                                -123  -31 -114   91 """, 400, 30)



#!/usr/bin/env python2.4

"""
Takes a list of maf filenames on the command line and prints a comma separated
list of the species that occur in all of the mafs.

usage %prog maf1 maf2 ...
"""
from __future__ import print_function

import operator
import sys

from six.moves import reduce

import bx.align.maf

files = sys.argv[1:]
sets = []

for file in files:
    sys.stderr.write(".")
    s = set()
    for block in bx.align.maf.Reader(open(file)):
        for comp in block.components:
            s.add(comp.src.split('.')[0])
    sets.append(s)

inter = reduce(operator.and_, sets)
print(",".join(inter))



#!/usr/bin/env python

"""
Tool for filtering a tabular data file. Fields are separated by tabs, the
header line is denoted by a '#' in the first byte, comments are denoted by
a '#' at the start of any subsequent line.

Expressions can use column names as well as numbers. The -c options allows
cutting, again using field name or numbers.

usage: %prog expression < table
    -H, --header:       keep header in output
    -C, --comments:     keep comments in output
    --force-header:     assume the first line is a header even if it does not start with "#"
    -c, --cols=1,2:     names or indexes of columns to keep
"""
from __future__ import print_function

import sys

import bx.tabular.io
from bx.cookbook import doc_optparse


def __main__():

    # Parse command line arguments
    options, args = doc_optparse.parse(__doc__)
    try:
        keep_header = bool(options.header)
        keep_comments = bool(options.comments)
        cols = []
        if options.cols:
            for c in options.cols.split(','):
                try:
                    v = int(c)
                except ValueError:
                    v = c
                cols.append(v)
        if len(args) > 0:
            expr = args[0]
        else:
            expr = None
        if options.force_header:
            force_header = bx.tabular.io.FIRST_LINE_IS_HEADER
        else:
            force_header = None
    except Exception:
        doc_optparse.exception()

    # Compile expression for SPEED
    if expr:
        expr = compile(expr, '<expr arg>', 'eval')

    for element in bx.tabular.io.TableReader(sys.stdin, force_header=force_header):
        if isinstance(element, bx.tabular.io.Header):
            if keep_header:
                if cols:
                    print("#" + "\t".join(element[c] for c in cols))
                else:
                    print(element)
        elif isinstance(element, bx.tabular.io.Comment):
            if keep_comments:
                print(element)
        else:
            if expr is None or bool(eval(expr, dict(row=element))):
                if cols:
                    print("\t".join(element[c] for c in cols))
                else:
                    print(element)


if __name__ == "__main__":
    __main__()



#!/usr/bin/python

"""
Returns a bed-like translation of a CDS in which each record corresponds to
a single site in the CDS and includes additional fields for site degenaracy,
position ind CDS, and amino acid encoded.

usage: %prog nibdir genefile [options]
    -o, --outfile=o:      output file
    -f, --format=f:       format bed (default), or gtf|gff
    -a, --allpositions: 1st, 2nd and 3rd positions are evaluated for degeneracy given the sequence at the other two positions.  Many 1d sites in 1st codon positions become 2d sites when considered this way.
    -n, --include_name: include the 'name' or 'id' field from the source file on every line of output
"""
from __future__ import print_function

import os
import re
import string
import sys

from bx.cookbook import doc_optparse
from bx.gene_reader import CDSReader
from bx.seq import nib

GENETIC_CODE = """
TTT (Phe/F)Phenylalanine
TTC (Phe/F)Phenylalanine
TTA (Leu/L)Leucine
TTG (Leu/L)Leucine, Start
TCT (Ser/S)Serine
TCC (Ser/S)Serine
TCA (Ser/S)Serine
TCG (Ser/S)Serine
TAT (Tyr/Y)Tyrosine
TAC (Tyr/Y)Tyrosine
TAA Ochre (Stop)
TAG Amber (Stop)
TGT (Cys/C)Cysteine
TGC (Cys/C)Cysteine
TGA Opal (Stop)
TGG (Trp/W)Tryptophan
CTT (Leu/L)Leucine
CTC (Leu/L)Leucine
CTA (Leu/L)Leucine
CTG (Leu/L)Leucine, Start
CCT (Pro/P)Proline
CCC (Pro/P)Proline
CCA (Pro/P)Proline
CCG (Pro/P)Proline
CAT (His/H)Histidine
CAC (His/H)Histidine
CAA (Gln/Q)Glutamine
CAG (Gln/Q)Glutamine
CGT (Arg/R)Arginine
CGC (Arg/R)Arginine
CGA (Arg/R)Arginine
CGG (Arg/R)Arginine
ATT (Ile/I)Isoleucine, Start2
ATC (Ile/I)Isoleucine
ATA (Ile/I)Isoleucine
ATG (Met/M)Methionine, Start1
ACT (Thr/T)Threonine
ACC (Thr/T)Threonine
ACA (Thr/T)Threonine
ACG (Thr/T)Threonine
AAT (Asn/N)Asparagine
AAC (Asn/N)Asparagine
AAA (Lys/K)Lysine
AAG (Lys/K)Lysine
AGT (Ser/S)Serine
AGC (Ser/S)Serine
AGA (Arg/R)Arginine
AGG (Arg/R)Arginine
GTT (Val/V)Valine
GTC (Val/V)Valine
GTA (Val/V)Valine
GTG (Val/V)Valine, Start2
GCT (Ala/A)Alanine
GCC (Ala/A)Alanine
GCA (Ala/A)Alanine
GCG (Ala/A)Alanine
GAT (Asp/D)Aspartic acid
GAC (Asp/D)Aspartic acid
GAA (Glu/E)Glutamic acid
GAG (Glu/E)Glutamic acid
GGT (Gly/G)Glycine
GGC (Gly/G)Glycine
GGA (Gly/G)Glycine
GGG (Gly/G)Glycine
"""


def translate(codon, genetic_code):
    c1, c2, c3 = codon
    return genetic_code[c1][c2][c3]


""" parse the doc string to hash the genetic code"""
GEN_CODE = {}
for line in GENETIC_CODE.split('\n'):
    if line.strip() == '':
        continue
    f = re.split(r'\s|\(|\)|\/', line)
    codon = f[0]
    c1, c2, c3 = codon
    aminoacid = f[3]
    if c1 not in GEN_CODE:
        GEN_CODE[c1] = {}
    if c2 not in GEN_CODE[c1]:
        GEN_CODE[c1][c2] = {}

    GEN_CODE[c1][c2][c3] = aminoacid


def getnib(nibdir):
    seqs = {}
    for nibf in os.listdir(nibdir):
        if not nibf.endswith('.nib'):
            continue
        chr = nibf.replace('.nib', '')
        file = os.path.join(nibdir, nibf)
        seqs[chr] = nib.NibFile(open(file))

    return seqs


REVMAP = string.maketrans("ACGTacgt", "TGCAtgca")


def revComp(seq):
    return seq[::-1].translate(REVMAP)


def Comp(seq):
    return seq.translate(REVMAP)


def main():

    options, args = doc_optparse.parse(__doc__)
    try:
        if options.outfile:
            out = open(options.outfile, "w")
        else:
            out = sys.stdout
        if options.format:
            format = options.format
        else:
            format = 'bed'

        allpositions = bool(options.allpositions)
        include_name = bool(options.include_name)
        nibdir = args[0]
        bedfile = args[1]
    except Exception:
        doc_optparse.exit()

    nibs = getnib(nibdir)

    for chrom, strand, cds_exons, name in CDSReader(open(bedfile), format=format):

        cds_seq = ''

        # genome_seq_index maps the position in CDS to position on the genome
        genome_seq_index = []
        for (c_start, c_end) in cds_exons:
            cds_seq += nibs[chrom].get(c_start, c_end-c_start)
            for i in range(c_start, c_end):
                genome_seq_index.append(i)

        cds_seq = cds_seq.upper()

        if strand == '+':
            frsts = range(0, len(cds_seq), 3)
            offsign = 1
        else:
            cds_seq = Comp(cds_seq)
            frsts = range(2, len(cds_seq), 3)
            offsign = -1

        offone = 1 * offsign
        offtwo = 2 * offsign

        all = ['A', 'C', 'G', 'T']

        for first_pos in frsts:
            c1 = first_pos
            c2 = first_pos + offone
            c3 = first_pos + offtwo
            try:
                assert c3 < len(cds_seq)
            except AssertionError:
                print("out of sequence at %d for %s, %d" % (c3, chrom, genome_seq_index[first_pos]), file=sys.stderr)
                continue
            codon = cds_seq[c1], cds_seq[c2], cds_seq[c3]
            aa = translate(codon, GEN_CODE)
            degeneracy3 = str(list(GEN_CODE[codon[0]][codon[1]].values()).count(aa)) + "d"

            if not include_name:
                name_text = ''
            else:
                name_text = name.replace(' ', '_')

            if allpositions:
                try:
                    degeneracy1 = str([GEN_CODE[k][codon[1]][codon[2]] for k in all].count(aa)) + "d"
                    degeneracy2 = str([GEN_CODE[codon[0]][k][codon[2]] for k in all].count(aa)) + "d"
                except TypeError as s:
                    print(list(GEN_CODE.values()), file=sys.stderr)
                    raise TypeError(s)

                if strand == '+':
                    print(chrom, genome_seq_index[c1], genome_seq_index[c1] + 1, cds_seq[c1], degeneracy1, aa, name_text, file=out)
                    print(chrom, genome_seq_index[c2], genome_seq_index[c2] + 1, cds_seq[c2], degeneracy2, aa, name_text, file=out)
                    print(chrom, genome_seq_index[c3], genome_seq_index[c3] + 1, cds_seq[c3], degeneracy3, aa, name_text, file=out)
                else:
                    print(chrom, genome_seq_index[c3], genome_seq_index[c3] + 1, cds_seq[c3], degeneracy3, aa, name_text, file=out)
                    print(chrom, genome_seq_index[c2], genome_seq_index[c2] + 1, cds_seq[c2], degeneracy2, aa, name_text, file=out)
                    print(chrom, genome_seq_index[c1], genome_seq_index[c1] + 1, cds_seq[c1], degeneracy1, aa, name_text, file=out)
            else:
                if strand == '+':
                    for b in c1, c2:
                        print(chrom, genome_seq_index[b], genome_seq_index[b] + 1, cds_seq[b], "1d", aa, name_text, file=out)
                    print(chrom, genome_seq_index[c3], genome_seq_index[c3] + 1, cds_seq[c3], degeneracy3, aa, name_text, file=out)
                else:
                    print(chrom, genome_seq_index[c3], genome_seq_index[c3] + 1, cds_seq[c3], degeneracy3, aa, name_text, file=out)
                    for b in c2, c1:
                        print(chrom, genome_seq_index[b], genome_seq_index[b] + 1, cds_seq[b], "1d", aa, name_text, file=out)
    out.close()


if __name__ == '__main__':
    main()



"""
Tests for `bx.align.score`.
"""
import unittest

from numpy import (
    allclose,
    array,
    cumsum,
)
from six import StringIO

import bx.align.maf
import bx.align.score

aligns = [("CCACTAGTTTTTAAATAATCTACTATCAAATAAAAGATTTGTTAATAATAAATTTTAAATCATTAACACTT",
           "CCATTTGGGTTCAAAAATTGATCTATCA----------TGGTGGATTATTATTTAGCCATTAAGGACAAAT",
           -111),
          ("CCACTAGTTTTTAAATAATCTAC-----AATAAAAGATTTGTTAATAAT---AAATTTTAAATCATTAA-----CACTT",
           "CCATTTGGGTTCAAAAATTGATCTATCA----------TGGTGGAT---TATTATTT-----AGCCATTAAGGACAAAT",
           -3626),
          ("CCACTAGTTTTTGATTC",
           "CCATTTGGGTTC-----",
           -299),
          ("CTTAGTTTTTGATCACC",
           "-----CTTGGGTTTACC",
           -299),
          ("gggaattgaacaatgagaacacatggacacaggaaggggaacatcacacacc----------ggggcctgttgtggggtggggggaag",
           "ggaactagaacaagggagacacatacaaacaacaacaacaacaacacagcccttcccttcaaagagcttatagtctgatggaggagag",
           1690)]

mafs = """##maf
a score=2883.0
s hg17.chr1             6734 30 + 245522847 CTACCTCAGTGTGGAAGGTGGGCAGTTCTG
s rheMac1.SCAFFOLD71394 9319 30 -     13789 CTACCTCAGTGTGGAAGGTGGGCAGTTCTG

a score=8167.0
s hg17.chr1             41401 40 + 245522847 TGTGTGATTAATGCCTGAGACTGTGTGAAGTAAGAGATGG
s panTro1.chr1          49673 40 + 229575298 TGCGTGATTAATGCCTGAGATTGTGTGAAGTAAAAGATGG
s rheMac1.SCAFFOLD45837 26063 33 -     31516 TGTGTGATTAATGCCTGAGATTGTGTGAAGTAA-------
"""

nonsymm_scheme = bx.align.score.build_scoring_scheme("""  A    C    G    T
                                                           91    0  -31 -123
                                                         -114  100 -125  -31
                                                          -31 -125  100 -114
                                                         -123  -31 -114   91 """, 400, 30)

aligns_for_nonsymm_scheme = [("AAAACCCCGGGGTTTT",
                              "ACGTACGTACGTACGT",
                              -580)]

asymm_scheme = bx.align.score.build_scoring_scheme("""    01   02    A    C    G    T
                                                       01  200 -200  -50  100  -50  100
                                                       02 -200  200  100  -50  100  -50 """,
                                                   0, 0, gap1='\x00')

aligns_for_asymm_scheme = [("\x01\x01\x01\x01\x01\x01",
                            "ACGT\x01\x02",
                            100)]


class BasicTests(unittest.TestCase):

    def test_scoring_text(self):
        ss = bx.align.score.hox70
        for t1, t2, score in aligns:
            self.assertEqual(bx.align.score.score_texts(ss, t1, t2), score)

    def test_align(self):
        ss = bx.align.score.hox70
        for block in bx.align.maf.Reader(StringIO(mafs)):
            self.assertEqual(bx.align.score.score_alignment(ss, block), float(block.score))

    def test_accumulate(self):
        ss = bx.align.score.hox70
        self.assert_(allclose(bx.align.score.accumulate_scores(ss, "-----CTTT", "CTTAGTTTA"),
                              cumsum(array([-430, -30, -30, -30, -30, -31, 91, 91, -123]))))
        self.assert_(allclose(bx.align.score.accumulate_scores(ss, "-----CTTT", "CTTAGTTTA", skip_ref_gaps=True),
                              cumsum(array([-581, 91, 91, -123]))))

    def test_nonsymm_scoring(self):
        ss = nonsymm_scheme
        for t1, t2, score in aligns_for_nonsymm_scheme:
            self.assertEqual(bx.align.score.score_texts(ss, t1, t2), score)

    def test_asymm_scoring(self):
        ss = asymm_scheme
        for t1, t2, score in aligns_for_asymm_scheme:
            self.assertEqual(bx.align.score.score_texts(ss, t1, t2), score)



"""
Semi-random access to bz2 compressed data.
"""

import struct
from io import BytesIO

from six import Iterator

try:
    import pkg_resources
    pkg_resources.require('python_lzo')
    import lzo
except Exception:
    pass

from bx_extras import lrucache


class SeekableLzopFile(Iterator):
    """
    Filelike object supporting read-only semi-random access to bz2 compressed
    files for which an offset table (bz2t) has been generated by `bzip-table`.
    """

    def __init__(self, filename, table_filename, block_cache_size=0, **kwargs):
        self.filename = filename
        self.table_filename = table_filename
        self.init_table()
        self.file = open(self.filename, "rb")
        self.dirty = True
        self.at_eof = False
        self.file_pos = 0
        self.current_block_index = -1
        self.current_block = None
        if block_cache_size > 0:
            self.cache = lrucache.LRUCache(block_cache_size)
        else:
            self.cache = None

    def init_table(self):
        self.block_size = None
        self.block_info = []
        # Position of corresponding block in compressed file (in bytes)
        for line in open(self.table_filename):
            fields = line.split()
            if fields[0] == "s":
                self.block_size = int(fields[1])
            if fields[0] == "o":
                offset = int(fields[1])
                compressed_size = int(fields[2])
                size = int(fields[3])
                self.block_info.append((offset, compressed_size, size))
        self.nblocks = len(self.block_info)

    def close(self):
        self.file.close()

    def load_block(self, index):
        if self.cache is not None and index in self.cache:
            return self.cache[index]
        else:
            offset, csize, size = self.block_info[index]
            # Get the block of compressed data
            self.file.seek(offset)
            data = self.file.read(csize)
            # Need to prepend a header for python-lzo module (silly)
            data = b''.join((b'\xf0', struct.pack("!I", size), data))
            value = lzo.decompress(data)
            if self.cache is not None:
                self.cache[index] = value
            return value

    def fix_dirty(self):
        chunk, offset = self.get_block_and_offset(self.file_pos)
        if self.current_block_index != chunk:
            self.current_block = BytesIO(self.load_block(chunk))
            self.current_block.read(offset)
            self.current_block_index = chunk
        else:
            self.current_block.seek(offset)
        self.dirty = False

    def get_block_and_offset(self, index):
        return int(index // self.block_size), int(index % self.block_size)

    def seek(self, offset, whence=0):
        """
        Move the file pointer to a particular offset.
        """
        # Determine absolute target position
        if whence == 0:
            target_pos = offset
        elif whence == 1:
            target_pos = self.file_pos + offset
        elif whence == 2:
            raise Exception("seek from end not supported")
        else:
            raise Exception("Invalid `whence` argument: %r", whence)
        # Check if this is a noop
        if target_pos == self.file_pos:
            return
        # Verify it is valid
        # Move the position
        self.file_pos = target_pos
        # Mark as dirty, the next time a read is done we need to actually
        # move the position in the bzip2 file
        self.dirty = True

    def tell(self):
        return self.file_pos

    def readline(self):
        if self.dirty:
            self.fix_dirty()
        if self.at_eof:
            return b""
        rval = []
        while True:
            line = self.current_block.readline()
            self.file_pos += len(line)
            rval.append(line)
            if len(line) > 0 and line[-1] == b'\n':
                break
            elif self.current_block_index == self.nblocks - 1:
                self.at_eof = True
                break
            else:
                self.current_block_index += 1
                self.current_block = BytesIO(self.load_block(self.current_block_index))
        return b"".join(rval)

    def __next__(self):
        line = self.readline()
        if line == b"":
            raise StopIteration

    def __iter__(self):
        return self

# --- Factor out ---


MAGIC = b"\x89\x4c\x5a\x4f\x00\x0d\x0a\x1a\x0a"

F_ADLER32_D = 0x00000001
F_ADLER32_C = 0x00000002
F_H_EXTRA_FIELD = 0x00000040
F_H_GMTDIFF = 0x00000080
F_CRC32_D = 0x00000100
F_CRC32_C = 0x00000200
F_MULTIPART = 0x00000400
F_H_FILTER = 0x00000800
F_H_CRC32 = 0x00001000

assert struct.calcsize("!H") == 2
assert struct.calcsize("!I") == 4


class UnpackWrapper(object):
    def __init__(self, file):
        self.file = file

    def read(self, amt):
        return self.file.read(amt)

    def get(self, fmt):
        t = struct.unpack(fmt, self.file.read(struct.calcsize(fmt)))
        return t[0]



"""
Compute the intersection of two sets of genomic intervals, either base-by-base
or at the interval level. The returned GenomicIntervalReader will be in
the order of the first set of intervals passed in, with the corresponding
additional fields.
"""

from bx.intervals.io import (
    BitsetSafeReaderWrapper,
    GenomicInterval
)
from bx.intervals.operations import bits_set_in_range
from bx.tabular.io import (
    Comment,
    Header,
)


def intersect(readers, mincols=1, upstream_pad=0, downstream_pad=0, pieces=True, lens={}, comments=True):
    # The incoming lens dictionary is a dictionary of chromosome lengths which are used to initialize the bitsets.
    # Read all but first into bitsets and intersect to one
    primary = readers[0]
    intersect = readers[1:]
    # Handle any ValueError, IndexError and OverflowError exceptions that may be thrown when
    # the bitsets are being created by skipping the problem lines
    intersect[0] = BitsetSafeReaderWrapper(intersect[0], lens=lens)
    bitsets = intersect[0].binned_bitsets(upstream_pad=upstream_pad, downstream_pad=downstream_pad, lens=lens)
    intersect = intersect[1:]
    for andset in intersect:
        bitset2 = andset.binned_bitsets(upstream_pad=upstream_pad, downstream_pad=downstream_pad, lens=lens)
        for chrom in bitsets:
            if chrom not in bitset2:
                continue
            bitsets[chrom].iand(bitset2[chrom])
        intersect = intersect[1:]

    # Read remaining intervals and intersect
    for interval in primary:
        if isinstance(interval, Header):
            yield interval
        if isinstance(interval, Comment) and comments:
            yield interval
        elif isinstance(interval, GenomicInterval):
            chrom = interval.chrom
            start = int(interval.start)
            end = int(interval.end)
            if chrom not in bitsets:
                continue
            if start > end:
                try:
                    # This will only work if primary is a NiceReaderWrapper
                    primary.skipped += 1
                    # no reason to stuff an entire bad file into memmory
                    if primary.skipped < 10:
                        primary.skipped_lines.append((primary.linenum, primary.current_line, "Interval start after end!"))
                except Exception:
                    pass
                continue
            out_intervals = []
            # Intersect or Overlap
            try:
                if bitsets[chrom].count_range(start, end-start) >= mincols:
                    if pieces:
                        out_intervals = bits_set_in_range(bitsets[chrom], start, end)
                    else:
                        out_intervals = [(start, end)]
                # Write the intervals
                for start, end in out_intervals:
                    new_interval = interval.copy()
                    new_interval.start = start
                    new_interval.end = end
                    yield new_interval
            except IndexError as e:
                try:
                    # This will only work if primary is a NiceReaderWrapper
                    primary.skipped += 1
                    # no reason to stuff an entire bad file into memmory
                    if primary.skipped < 10:
                        primary.skipped_lines.append((primary.linenum, primary.current_line, str(e)))
                except Exception:
                    pass
                continue



"""
Support for reading and writing the LAV format produced by the `blastz`_
pairwise aligner.

.. _blastz: http://www.bx.psu.edu/miller_lab/
"""
from __future__ import print_function

import sys

from six import Iterator, StringIO

import bx.seq
from bx.align import (
    Alignment,
    Component,
    src_merge,
    src_split
)


class Reader(Iterator):
    """Iterate over all lav blocks in a file in order"""

    def __init__(self, file, path_subs=None, fail_to_ns=False):
        self.file = file
        self.lineNumber = 0
        self.path_subs = path_subs   # list of (prefix,replacement) to allow
        if self.path_subs is None:  # .. redirection of sequence file paths
            self.path_subs = []       # .. on different machines
        self.fail_to_ns = fail_to_ns  # True => if sequences fail to open, create a fake file of all Ns

        self.d_stanza_text = None

        self.seq1_filename = None
        self.seq1_file = None
        self.seq1_header = None
        self.seq1_start = None
        self.seq1_end = None
        self.seq1_strand = None
        self.seq1_contig = None
        self.seq1_src = None
        self.seq1_gap = None

        self.seq2_filename = None
        self.seq2_file = None
        self.seq2_header = None
        self.seq2_start = None
        self.seq2_end = None
        self.seq2_strand = None
        self.seq2_contig = None
        self.seq2_src = None
        self.seq2_gap = None

    def __next__(self):
        while (True):
            line = self.fetch_line(strip=None, requireLine=False)
            assert (line), "unexpected end of file (missing #:eof)"
            line = line.rstrip()
            if line == "":  # (allow blank lines between stanzas)
                continue
            if line == "#:eof":
                line = self.file.readline().rstrip()
                assert (not line), "extra line after #:eof (line %d, \"%s\")" % (self.lineNumber, line)
                return None
            if line == "#:lav":
                continue
            if line.startswith("d {"):
                self.d_stanza_text = self.parse_unknown_stanza()
                continue
            if line.startswith("s {"):
                self.parse_s_stanza()
                continue
            if line.startswith("h {"):
                self.parse_h_stanza()
                continue
            if line.startswith("a {"):
                (score, pieces) = self.parse_a_stanza()
                break
            if line.endswith("{"):
                self.parse_unknown_stanza()
                continue
            assert (False), "incomprehensible line (line %d, \"%s\")" % (self.lineNumber, line)
        return self.build_alignment(score, pieces)

    def __iter__(self):
        return ReaderIter(self)

    def close(self):
        self.file.close()

    def open_seqs(self):
        if self.seq1_file is not None and self.seq2_file is not None:
            return

        if self.seq1_file is None:
            if self.seq1_strand == "+":
                revcomp = False
            else:
                revcomp = "-5'"
            if self.seq1_contig == 1:
                contig = None
            else:
                contig = self.seq1_contig
            try:
                f = open(self.seq1_filename, "rb")
            except Exception:
                if self.fail_to_ns:
                    f = StringIO(">seq1\n" + ("n" * (self.seq1_end - self.seq1_start)))
                    revcomp = False
                    contig = 1
                else:
                    assert (False), "failed to open %s" % self.seq1_filename
            self.seq1_file = bx.seq.seq_file(f, revcomp=revcomp, contig=contig)
            self.seq1_gap = self.seq1_file.gap
            try:
                name1 = self.header_to_src_name(self.seq1_header)
            except ValueError:
                try:
                    name1 = self.path_to_src_name(self.seq1_filename)
                except ValueError:
                    name1 = "seq1"
            (species1, chrom1) = src_split(name1)
            self.seq1_src = src_merge(species1, chrom1, contig)
            if contig is not None:
                chrom1 += "[%s]" % contig

        if self.seq2_file is None:
            if self.seq2_strand == "+":
                revcomp = False
            else:
                revcomp = "-5'"
            if self.seq2_contig == 1:
                contig = None
            else:
                contig = self.seq2_contig
            try:
                f = open(self.seq2_filename, "rb")
            except Exception:
                if self.fail_to_ns:
                    f = StringIO(">seq2\n" + ("n" * (self.seq2_end - self.seq2_start)))
                    revcomp = False
                    contig = 1
                else:
                    assert (False), "failed to open %s" % self.seq1_filename
            self.seq2_file = bx.seq.seq_file(f, revcomp=revcomp, contig=contig)
            self.seq2_gap = self.seq2_file.gap
            try:
                name2 = self.header_to_src_name(self.seq2_header)
            except ValueError:
                try:
                    name2 = self.path_to_src_name(self.seq2_filename)
                except ValueError:
                    name2 = "seq2"
            (species2, chrom2) = src_split(name2)
            self.seq2_src = src_merge(species2, chrom2, contig)
            if contig is not None:
                chrom2 += "[%s]" % contig

        length1 = self.seq1_file.length
        length2 = self.seq2_file.length
        assert (species1 != species2) or (chrom1 != chrom2) or (length1 == length2), \
            "conflicting lengths for %s (%d and %d)" % (self.seq1_src, length1, length2)

        self.species_to_lengths = {}
        self.species_to_lengths[species1] = {}
        self.species_to_lengths[species2] = {}  # (OK if it clobbers line above)
        self.species_to_lengths[species1][chrom1] = self.seq1_file.length
        self.species_to_lengths[species2][chrom2] = self.seq2_file.length

    def close_seqs(self):
        if self.seq1_file is not None:
            self.seq1_file.close()
            self.seq1_file = None
        if self.seq2_file is not None:
            self.seq2_file.close()
            self.seq2_file = None

    def parse_s_stanza(self):
        self.close_seqs()
        line = self.fetch_line(report=" in s-stanza")
        (self.seq1_filename,
         self.seq1_start,
         self.seq1_end,
         self.seq1_strand,
         self.seq1_contig) = self.parse_s_seq(line)

        line = self.fetch_line(report=" in s-stanza")
        (self.seq2_filename,
         self.seq2_start,
         self.seq2_end,
         self.seq2_strand,
         self.seq2_contig) = self.parse_s_seq(line)

        line = self.fetch_line(report=" in s-stanza")
        assert (line == "}"), "improper s-stanza terminator (line %d, \"%s\")" \
            % (self.lineNumber, line)

    def parse_s_seq(self, line):
        fields = line.split()
        filename = fields[0].strip('"')
        start = int(fields[1]) - 1
        end = int(fields[2])
        contig = int(fields[4])
        if fields[3] == "1":
            strand = "-"
        else:
            strand = "+"
        if filename.endswith("-"):
            assert (strand == "-"), "strand mismatch in \"%s\"" % line
            filename = filename[:-1]
        filename = do_path_subs(filename, self.path_subs)
        return (filename, start, end, strand, contig)

    def parse_h_stanza(self):
        line = self.fetch_line(strip='"', report=" in h-stanza")
        self.seq1_header = line
        self.seq1_header_prefix = ""
        if line.startswith(">"):
            self.seq1_header = line[1:].strip()
            self.seq1_header_prefix = ">"
        self.seq1_header = self.seq1_header.split(None, 1)
        if len(self.seq1_header) > 0:
            self.seq1_header = self.seq1_header[0]
        else:
            self.seq1_header = "seq1"

        line = self.fetch_line(strip='"', report=" in h-stanza")
        self.seq2_header = line
        self.seq2_header_prefix = ""
        if line.startswith(">"):
            self.seq2_header = line[1:].strip()
            self.seq2_header_prefix = ">"
        self.seq2_header = self.seq2_header.split(None, 1)
        if len(self.seq2_header) > 0:
            self.seq2_header = self.seq2_header[0]
        else:
            self.seq2_header = "seq2"

        line = self.fetch_line(report=" in h-stanza")
        assert (line == "}"), "improper h-stanza terminator (line %d, \"%s\")" \
            % (self.lineNumber, line)

    def parse_a_stanza(self):
        """returns the pair (score,pieces)
           where pieces is a list of ungapped segments (start1,start2,length,pctId)
           with start1,start2 origin-0"""
        # 's' line -- score, 1 field
        line = self.fetch_line(report=" in a-stanza")
        fields = line.split()
        assert (fields[0] == "s"), "s line expected in a-stanza (line %d, \"%s\")" \
            % (self.lineNumber, line)
        try:
            score = int(fields[1])
        except ValueError:
            score = float(fields[1])

        # 'b' line -- begin positions in seqs, 2 fields
        line = self.fetch_line(report=" in a-stanza")
        fields = line.split()
        assert (fields[0] == "b"), "b line expected in a-stanza (line %d, \"%s\")" \
            % (self.lineNumber, line)

        # 'e' line -- end positions in seqs, 2 fields
        line = self.fetch_line(report=" in a-stanza")
        fields = line.split()
        assert (fields[0] == "e"), "e line expected in a-stanza (line %d, \"%s\")" \
            % (self.lineNumber, line)

        # 'l' lines
        pieces = []
        while (True):
            line = self.fetch_line(report=" in a-stanza")
            fields = line.split()
            if fields[0] != "l":
                break
            start1 = int(fields[1]) - 1
            start2 = int(fields[2]) - 1
            length = int(fields[3]) - start1
            length2 = int(fields[4]) - start2
            try:
                pctId = int(fields[5])
            except ValueError:
                pctId = float(fields[5])
            assert (length2 == length), "length mismatch in a-stanza"
            pieces.append((start1+self.seq1_start, start2+self.seq2_start, length, pctId))
        assert (line == "}"), "improper a-stanza terminator (line %d, \"%s\")" \
            % (self.lineNumber, line)
        return (score, pieces)

    def parse_unknown_stanza(self):
        lines = []
        while (True):
            line = self.fetch_line()
            assert (line), "unexpected end of file (missing #:eof)"
            if line == "}":
                break
            lines.append(line)
        return "  " + "\n  ".join(lines) + "\n"

    def fetch_line(self, strip=True, requireLine=True, report=""):
        if strip is None:
            line = self.file.readline()
        elif strip is True:
            line = self.file.readline().strip()
        else:
            line = self.file.readline().strip().strip(strip)
        self.lineNumber += 1
        if requireLine:
            assert (line), "unexpected blank line or end of file%s (line %d)" \
                % (report, self.lineNumber)
        return line

    def d_stanza(self):
        if self.d_stanza_text is None:
            return ""
        return "d {\n%s}" % self.d_stanza_text

    def s_stanza(self):
        if self.seq1_filename is None:
            return ""

        if self.seq1_strand == "-":
            seq1_strand = "1"
        else:
            seq1_strand = "0"
        if self.seq2_strand == "-":
            seq2_strand = "1"
        else:
            seq2_strand = "0"

        s = "  \"%s\" %d %d %s %d\n"\
            % (self.seq1_filename, self.seq2_start+1, self.seq1_end,
               seq1_strand, self.seq1_contig)
        s += "  \"%s\" %d %d %s %d\n"\
            % (self.seq2_filename, self.seq2_start+1, self.seq2_end,
               seq2_strand, self.seq2_contig)

        return "s {\n%s}" % s

    def h_stanza(self):
        if self.seq1_header is None:
            return ""
        s = "  \"%s%s\"\n" % (self.seq1_header_prefix, self.seq1_header)
        s += "  \"%s%s\"\n" % (self.seq2_header_prefix, self.seq2_header)
        return "h {\n%s}" % s

    def build_alignment(self, score, pieces):
        """converts a score and pieces to an alignment"""
        # build text
        self.open_seqs()
        text1 = text2 = ""
        end1 = end2 = None
        for (start1, start2, length, pctId) in pieces:
            if end1 is not None:
                if start1 == end1:  # insertion in sequence 2
                    text1 += self.seq1_gap * (start2-end2)
                    text2 += self.seq2_file.get(end2, start2-end2)
                else:  # insertion in sequence 1
                    text1 += self.seq1_file.get(end1, start1-end1)
                    text2 += self.seq2_gap * (start1-end1)

            text1 += self.seq1_file.get(start1, length)
            text2 += self.seq2_file.get(start2, length)
            end1 = start1 + length
            end2 = start2 + length
        # create alignment
        start1 = pieces[0][0]
        start2 = pieces[0][1]
        end1 = pieces[-1][0] + pieces[-1][2]
        end2 = pieces[-1][1] + pieces[-1][2]
        size1 = end1 - start1
        size2 = end2 - start2
        a = Alignment(score=score, species_to_lengths=self.species_to_lengths)
        # if (self.seq1_strand == "-"): start1 = self.seq1_file.length - end1
        a.add_component(Component(self.seq1_src, start1, size1, self.seq1_strand, text=text1))
        # if (self.seq2_strand == "-"): start2 = self.seq2_file.length - end2
        a.add_component(Component(self.seq2_src, start2, size2, self.seq2_strand, text=text2))
        return a

    def path_to_src_name(self, path_name):
        # converts, e.g. ".../hg18/seq/chr13.nib" to "hg18.chr13"
        if path_name is None or path_name == "":
            raise ValueError
        if path_name.endswith(".nib"):
            path_name = path_name[:-4]
        if path_name.endswith(".fa"):
            path_name = path_name[:-3]
        if path_name.endswith(".fasta"):
            path_name = path_name[:-6]
        slash = path_name.rfind("/")
        if slash == -1:
            return path_name
        name = path_name[slash+1:]
        path_name = path_name[:slash]
        if path_name.endswith("/seq"):
            path_name = path_name[:-4]
        slash = path_name.rfind("/")
        if slash != -1:
            path_name = path_name[slash+1:]
        return path_name + "." + name

    def header_to_src_name(self, header):
        # converts, e.g. "hg18.chr13:115404472-117281897" to "hg18.chr13"
        if header is None or header == "":
            raise ValueError
        colon = header.rfind(":")
        if colon != -1:
            header = header[:colon]
        if "/" in header:
            raise ValueError
        if header.count(".") == 0:
            return header
        header = header.split(".")
        if header[0] == "" or header[1] == "":
            raise ValueError
        return ".".join(header)


class ReaderIter(Iterator):
    def __init__(self, reader):
        self.reader = reader

    def __iter__(self):
        return self

    def __next__(self):
        v = next(self.reader)
        if not v:
            raise StopIteration
        return v


class LavAsPiecesReader(Reader):
    """Iterate over all lav blocks in a file in order, returning alignments
       as score and pieces, as returned by Reader.parse_a_stanza"""

    def build_alignment(self, score, pieces):
        return (score, pieces)


class Writer(object):

    # blockHash is a hash from (src1,strand1,src2,strand2) to a list of blocks;
    # the blocks are collected on each call to write(), but the actual writing
    # does not occur until close().

    def __init__(self, file, attributes={}):
        self.file = file
        self.fname1 = None
        self.fname2 = None
        self.block = 0
        self.blockHash = {}  # (see note above)

        if "name_format_1" in attributes:
            self.fname1 = attributes["name_format_1"]
        if "name_format_2" in attributes:
            self.fname2 = attributes["name_format_2"]

        if "d_stanza" in attributes:
            self.write_lav_marker()
            print("d {", file=self.file)
            print(attributes["d_stanza"], file=self.file)
            print("}", file=self.file)

    def write(self, alignment):
        if len(alignment.components) != 2:
            raise ValueError(
                "%d-component alignment is not compatible with lav" %
                len(alignment.components))

        c1 = alignment.components[0]
        c2 = alignment.components[1]

        key = (c1.src, c1.strand, c2.src, c2.strand)
        if key not in self.blockHash:
            self.blockHash[key] = []
        self.blockHash[key].append(alignment)
        self.block += 1

    def close(self):
        keys = [key for key in self.blockHash]
        keys = sort_keys_by_chrom(keys)
        for key in keys:
            (src1, strand1, src2, strand2) = key
            alignment = self.blockHash[key][0]
            self.src1 = src1
            self.strand1 = strand1
            self.length1 = alignment.src_size(src1)
            self.src2 = src2
            self.strand2 = strand2
            self.length2 = alignment.src_size(src2)
            self.write_s_stanza()
            self.write_h_stanza()
            for alignment in self.blockHash[key]:
                self.write_a_stanza(alignment)
        self.write_trailer()
        if self.file != sys.stdout:
            self.file.close()

    def write_s_stanza(self):
        self.write_lav_marker()
        (strand1, flag1) = minus_or_nothing(self.strand1)
        (strand2, flag2) = minus_or_nothing(self.strand2)
        fname1 = build_filename(self.fname1, self.src1)
        fname2 = build_filename(self.fname2, self.src2)
        print("s {", file=self.file)
        print("  \"%s%s\" 1 %d %d 1" % (fname1, strand1, self.length1, flag1), file=self.file)
        print("  \"%s%s\" 1 %d %d 1" % (fname2, strand2, self.length2, flag2), file=self.file)
        print("}", file=self.file)

    def write_h_stanza(self):
        strand1 = rc_or_nothing(self.strand1)
        strand2 = rc_or_nothing(self.strand2)
        print("h {", file=self.file)
        print("  \"> %s%s\"" % (self.src1, strand1), file=self.file)
        print("  \"> %s%s\"" % (self.src2, strand2), file=self.file)
        print("}", file=self.file)

    def write_a_stanza(self, alignment):
        c1 = alignment.components[0]
        pos1 = c1.start
        text1 = c1.text.upper()
        c2 = alignment.components[1]
        pos2 = c2.start
        text2 = c2.text.upper()

        # collect ungapped pieces

        pieces = []
        piece1 = None

        for ix in range(len(text1)):
            ch1 = text1[ix]
            ch2 = text2[ix]

            nonGap = (ch1 != "-") and (ch2 != "-")
            if nonGap:
                if piece1 is None:  # new piece starts
                    (piece1, piece2, idCount) = (pos1, pos2, 0)
                if ch1 == ch2:
                    idCount += 1
            elif piece1 is not None:  # new gap starts
                size = pos1 - piece1
                pctId = (200*idCount + size) / (2*size)
                pieces.append((piece1, piece2, size, pctId))
                piece1 = None

            if ch1 != "-":
                pos1 += 1
            if ch2 != "-":
                pos2 += 1

        if piece1 is not None:
            size = pos1 - piece1
            pctId = (200*idCount + size) / (2*size)
            pieces.append((piece1, piece2, size, pctId))

        # write the block

        (start1, start2, size, pctId) = pieces[-1]  # get end of final piece
        end1 = start1 + size
        end2 = start2 + size

        (start1, start2, size, pctId) = pieces[0]   # get start of first piece

        score = int(round(alignment.score))

        print("a {", file=self.file)
        print("  s %s" % score, file=self.file)
        print("  b %d %d" % (start1 + 1, start2 + 1), file=self.file)
        print("  e %d %d" % (end1, end2), file=self.file)
        for (start1, start2, size, pctId) in pieces:
            print("  l %d %d %d %d %d" % (start1 + 1, start2 + 1, start1 + size, start2 + size, pctId), file=self.file)
        print("}", file=self.file)

    def write_lav_marker(self):
        print("#:lav", file=self.file)

    def write_trailer(self):
        print("#:eof", file=self.file)


def sort_keys_by_chrom(keys):
    decorated = sorted((chrom_key(src1), strand1, chrom_key(src2), strand2, (src1, strand1, src2, strand2))
                       for (src1, strand1, src2, strand2) in keys)
    return [key for (src1, strand1, src2, strand2, key) in decorated]


def chrom_key(src):
    (species, chrom) = src_split(src)
    if chrom.startswith("chr"):
        chrom = chrom[3:]
    try:
        chrom = int(chrom)
    except ValueError:
        pass
    return chrom


def build_filename(fmt, src):
    if fmt is None:
        return src
    num = fmt.count("%s")
    if num == 0:
        return fmt
    (species, chrom) = src_split(src)
    if num == 1:
        return fmt % chrom
    return fmt % (species, chrom)


def minus_or_nothing(strand):
    if strand == "-":
        return ("-", 1)
    else:
        return ("", 0)


def rc_or_nothing(strand):
    if strand == "-":
        return " (reverse complement)"
    else:
        return ""


def do_path_subs(path, path_subs):
    for (prefix, replacement) in path_subs:
        if path.startswith(prefix):
            return replacement + path[len(prefix):]
    return path



"""
Support for the `MAF`_ multiple sequence alignment format used by `multiz`_.

.. _MAF: http://genome.ucsc.edu/FAQ/FAQformat.html#format5
.. _multiz: http://www.bx.psu.edu/miller_lab/
"""
from io import TextIOWrapper

from six import (
    Iterator,
    PY3,
    StringIO,
)

from bx import interval_index_file
from bx.align import (
    Alignment,
    Component
)

MAF_INVERSE_STATUS = 'V'
MAF_INSERT_STATUS = 'I'
MAF_CONTIG_STATUS = 'C'
MAF_CONTIG_NESTED_STATUS = 'c'
MAF_NEW_STATUS = 'N'
MAF_NEW_NESTED_STATUS = 'n'
MAF_MAYBE_NEW_STATUS = 'S'
MAF_MAYBE_NEW_NESTED_STATUS = 's'
MAF_MISSING_STATUS = 'M'


class MAFIndexedAccess(interval_index_file.AbstractIndexedAccess):
    """
    Indexed access to a MAF file.
    """

    def read_at_current_offset(self, file, **kwargs):
        """
        Read the MAF block at the current position in `file` and return an
        instance of `Alignment`.
        """
        return read_next_maf(file, **kwargs)

    def open_data(self):
        data = super(MAFIndexedAccess, self).open_data()
        if PY3:
            return TextIOWrapper(data, encoding="ascii")
        return data


class MAFMultiIndexedAccess(interval_index_file.AbstractMultiIndexedAccess):
    """
    Indexed access to multiple MAF files.
    """
    indexed_access_class = MAFIndexedAccess


Indexed = MAFIndexedAccess
"""Deprecated: `MAFIndexedAccess` is also available under the name `Indexed`."""

MultiIndexed = MAFMultiIndexedAccess
"""Deprecated: `MAFMultiIndexedAccess` is also available under the name `MultiIndexed`."""


class Reader(Iterator):
    """
    Iterate over all maf blocks in a file in order
    """

    def __init__(self, file, **kwargs):
        self.file = file
        self.maf_kwargs = kwargs
        # Read and verify maf header, store any attributes
        fields = self.file.readline().split()
        if fields[0] != '##maf':
            raise Exception("File does not have MAF header")
        self.attributes = parse_attributes(fields[1:])

    def __next__(self):
        return read_next_maf(self.file, **self.maf_kwargs)

    def __iter__(self):
        return ReaderIter(self)

    def close(self):
        self.file.close()


class ReaderIter(Iterator):
    """
    Adapts a `Reader` to the iterator protocol.
    """

    def __init__(self, reader):
        self.reader = reader

    def __iter__(self):
        return self

    def __next__(self):
        v = next(self.reader)
        if not v:
            raise StopIteration
        return v


class Writer(object):

    def __init__(self, file, attributes={}):
        self.file = file
        # Write header, Webb's maf code wants version first, we accomodate
        if 'version' not in attributes:
            attributes['version'] = 1
        self.file.write("##maf version=%s" % attributes['version'])
        for key in attributes:
            if key == 'version':
                continue
            self.file.writelines(" %s=%s" % (key, attributes[key]))
        self.file.write("\n")

    def write(self, alignment):
        self.file.write("a score=" + str(alignment.score))
        for key in alignment.attributes:
            self.file.write(" %s=%s" % (key, alignment.attributes[key]))
        self.file.write("\n")
        # Components
        rows = []
        for c in alignment.components:
            # "Empty component" generates an 'e' row
            if c.empty:
                rows.append(("e", c.src, str(c.start), str(c.size), c.strand, str(c.src_size), c.synteny_empty))
                continue
            # Regular component
            rows.append(("s", c.src, str(c.start), str(c.size), c.strand, str(c.src_size), c.text))
            # If component has quality, write a q row
            if c.quality is not None:
                rows.append(("q", c.src, "", "", "", "", c.quality))
            # If component has synteny follow up with an 'i' row
            if c.synteny_left and c.synteny_right:
                rows.append(("i", c.src, "", "", "", "", " ".join(map(str, c.synteny_left + c.synteny_right))))
        self.file.write(format_tabular(rows, "llrrrrl"))
        self.file.write("\n")

    def close(self):
        self.file.close()

# ---- Helper methods -------------------------------------------------------


def from_string(string, **kwargs):
    return read_next_maf(StringIO(string), **kwargs)


def read_next_maf(file, species_to_lengths=None, parse_e_rows=False):
    """
    Read the next MAF block from `file` and return as an `Alignment`
    instance. If `parse_i_rows` is true, empty components will be created
    when e rows are encountered.
    """
    alignment = Alignment(species_to_lengths=species_to_lengths)
    # Attributes line
    line = readline(file, skip_blank=True)
    if not line:
        return None
    fields = line.split()
    if fields[0] != 'a':
        raise Exception("Expected 'a ...' line")
    alignment.attributes = parse_attributes(fields[1:])
    if 'score' in alignment.attributes:
        alignment.score = alignment.attributes['score']
        del alignment.attributes['score']
    else:
        alignment.score = 0
    # Sequence lines
    last_component = None
    while True:
        line = readline(file)
        # EOF or Blank line terminates alignment components
        if not line or line.isspace():
            break
        if line.isspace():
            break
        # Parse row
        fields = line.split()
        if fields[0] == 's':
            # An 's' row contains sequence for a component
            component = Component()
            component.src = fields[1]
            component.start = int(fields[2])
            component.size = int(fields[3])
            component.strand = fields[4]
            component.src_size = int(fields[5])
            if len(fields) > 6:
                component.text = fields[6].strip()
            # Add to set
            alignment.add_component(component)
            last_component = component
        elif fields[0] == 'e':
            # An 'e' row, when no bases align for a given species this tells
            # us something about the synteny
            if parse_e_rows:
                component = Component()
                component.empty = True
                component.src = fields[1]
                component.start = int(fields[2])
                component.size = int(fields[3])
                component.strand = fields[4]
                component.src_size = int(fields[5])
                component.text = None
                synteny = fields[6].strip()
                assert len(synteny) == 1, \
                    "Synteny status in 'e' rows should be denoted with a single character code"
                component.synteny_empty = synteny
                alignment.add_component(component)
                last_component = component
        elif fields[0] == 'i':
            # An 'i' row, indicates left and right synteny status for the
            # previous component, we hope ;)
            assert fields[1] == last_component.src, "'i' row does not follow matching 's' row"
            last_component.synteny_left = (fields[2], int(fields[3]))
            last_component.synteny_right = (fields[4], int(fields[5]))
        elif fields[0] == 'q':
            assert fields[1] == last_component.src, "'q' row does not follow matching 's' row"
            # TODO: Should convert this to an integer array?
            last_component.quality = fields[2]

    return alignment


def readline(file, skip_blank=False):
    """Read a line from provided file, skipping any blank or comment lines"""
    while True:
        line = file.readline()
        if not line:
            return None
        if line[0] != '#' and not (skip_blank and line.isspace()):
            return line


def parse_attributes(fields):
    """Parse list of key=value strings into a dict"""
    attributes = {}
    for field in fields:
        pair = field.split('=')
        attributes[pair[0]] = pair[1]
    return attributes


def format_tabular(rows, align=None):
    if len(rows) == 0:
        return ""
    lengths = [len(col) for col in rows[0]]
    for row in rows[1:]:
        for i in range(0, len(row)):
            lengths[i] = max(lengths[i], len(row[i]))
    rval = ""
    for row in rows:
        for i in range(0, len(row)):
            if align and align[i] == "l":
                rval += row[i].ljust(lengths[i])
            else:
                rval += row[i].rjust(lengths[i])
            rval += " "
        rval += "\n"
    return rval



"""
Pyrex/C extension for quickly finding potential CpG sites in pairs of 
sequences.
"""

from cpython.version cimport PY_MAJOR_VERSION

cdef extern from "find_cpg.h":
    int next_cpg( char * sp1, char * sp2, int start)
    int next_cpg_restricted( char * sp1, char *sp2, int start)
    int next_non_cpg( char * sp1, char * sp2, int start)

def find_cpg( sp1, sp2, start ):
    cdef char* a
    cdef char* b
    cdef int pos
    if PY_MAJOR_VERSION >= 3:
        bytes_sp1, bytes_sp2 = sp1.encode(), sp2.encode()
    else:
        bytes_sp1, bytes_sp2 = sp1, sp2
    a = bytes_sp1
    b = bytes_sp2
    pos = start
    if pos > len(sp1): return -1
    return next_cpg( a, b, pos )

def find_cpg_restricted( sp1, sp2, start ):
    cdef char* a
    cdef char* b
    cdef int pos
    if PY_MAJOR_VERSION >= 3:
        bytes_sp1, bytes_sp2 = sp1.encode(), sp2.encode()
    else:
        bytes_sp1, bytes_sp2 = sp1, sp2
    a = bytes_sp1
    b = bytes_sp2
    pos = start
    if pos > len(sp1): return -1
    return next_cpg_restricted( a, b, pos )

def find_non_cpg( sp1, sp2, start ):
    cdef char* a
    cdef char* b
    cdef int pos
    if PY_MAJOR_VERSION >= 3:
        bytes_sp1, bytes_sp2 = sp1.encode(), sp2.encode()
    else:
        bytes_sp1, bytes_sp2 = sp1, sp2
    a = bytes_sp1
    b = bytes_sp2
    pos = start
    if pos > len(sp1): return -1
    return next_non_cpg( a, b, pos )

def list_cpg( sp1, sp2 ):
    cdef char * a
    cdef char * b
    cdef int start
    if PY_MAJOR_VERSION >= 3:
        bytes_sp1, bytes_sp2 = sp1.encode(), sp2.encode()
    else:
        bytes_sp1, bytes_sp2 = sp1, sp2
    a = bytes_sp1
    b = bytes_sp2
    start = 0
    cpglist = list()
    while start > -1 and start < len(sp1):
        start = next_cpg( a, b, start )
        if start == -1: break
        cpglist.append(start)
        start = start + 1
    return cpglist

def list_cpg_restricted( sp1, sp2 ):
    cdef char * a
    cdef char * b
    cdef int start
    if PY_MAJOR_VERSION >= 3:
        bytes_sp1, bytes_sp2 = sp1.encode(), sp2.encode()
    else:
        bytes_sp1, bytes_sp2 = sp1, sp2
    a = bytes_sp1
    b = bytes_sp2
    start = 0
    cpglist = list()
    while start > -1 and start < len(sp1):
        start = next_cpg_restricted( a, b, start )
        if start == -1: break
        cpglist.append(start)
        start = start + 1
    return cpglist

def list_non_cpg( sp1, sp2 ):
    cdef char * a
    cdef char * b
    cdef int start
    if PY_MAJOR_VERSION >= 3:
        bytes_sp1, bytes_sp2 = sp1.encode(), sp2.encode()
    else:
        bytes_sp1, bytes_sp2 = sp1, sp2
    a = bytes_sp1
    b = bytes_sp2
    start = 0
    cpglist = list()
    while start > -1 and start < len(sp1):
        start = next_non_cpg( a, b, start )
        if start == -1: break
        cpglist.append(start)
        start = start + 1
    return cpglist

def remove_gaps( sp, cpglist ):
    for item in cpglist:
        if sp[item] == '-':
            cpglist.remove(item)
    return cpglist



#!/usr/bin/env python

"""
Find continuous regions that are covered by the first bed file (`bed_file_1`)
but not by the second bed file (`bed_file_2`)

usage: %prog bed_file_1 bed_file_2
"""
from __future__ import print_function


from bx.bitset_builders import binned_bitsets_from_file
from bx.cookbook import doc_optparse


def print_bits_as_bed(bits):
    end = 0
    while True:
        start = bits.next_set(end)
        if start == bits.size:
            break
        end = bits.next_clear(start)
        print("%s\t%d\t%d" % (chrom, start, end))


options, args = doc_optparse.parse(__doc__)
try:
    in_fname, in2_fname = args
except ValueError:
    doc_optparse.exit()

# Read first bed into some bitsets

bitsets1 = binned_bitsets_from_file(open(in_fname))
bitsets2 = binned_bitsets_from_file(open(in2_fname))

for chrom in bitsets1:
    if chrom not in bitsets1:
        continue
    bits1 = bitsets1[chrom]
    if chrom in bitsets2:
        bits2 = bitsets2[chrom]
        bits2.invert()
        bits1.iand(bits2)
    print_bits_as_bed(bits1)



#!/usr/bin/env python

"""
Remove any blocks from a maf that overlap any of a set of intervals.

usage: %prog interval files... < maf
"""

import sys

import bx.align.maf
from bx import intervals
from bx.cookbook import doc_optparse


def __main__():
    options, args = doc_optparse.parse(__doc__)

    try:
        assert len(args) > 0
    except AssertionError:
        doc_optparse.exit()

    # Load Intervals

    intersector = intervals.Intersecter()

    for f in args:
        for line in open(f):
            if line.startswith("#") or line.isspace():
                continue
            fields = line.split()
            intersector.add_interval(intervals.Interval(int(fields[0]), int(fields[1])))

    # Start MAF on stdout

    out = bx.align.maf.Writer(sys.stdout)

    # Iterate over input MAF

    for maf in bx.align.maf.Reader(sys.stdin):
        # Find overlap with reference component
        intersections = intersector.find(maf.components[0].start, maf.components[0].end)
        # Write only if no overlap
        if len(intersections) == 0:
            out.write(maf)

    # Close output MAF

    out.close()


if __name__ == "__main__":
    __main__()



"""
Extensions used by the `pwm` module.
"""

from cpython.version cimport PY_MAJOR_VERSION

cdef extern from "Python.h":
    int PyBytes_AsStringAndSize(object obj, char **buffer, Py_ssize_t* length) except -1

cdef extern from "numpy/arrayobject.h":
    ctypedef int intp
    ctypedef extern class numpy.ndarray [object PyArrayObject]:
        cdef char *data
        cdef int nd
        cdef intp *dimensions
        cdef intp *strides
        cdef int flags
    # These might be other types in the actual header depending on platform 
    ctypedef int npy_int16
    ctypedef float npy_float32

def score_string( ndarray matrix, ndarray char_to_index, object string, ndarray rval ):
    """
    Score each position in string `string` using the scoring matrix `matrix`.
    Characters in the string are mapped to columns in the matrix by `char_to_index`
    and the score for each position is stored in `rval`.
    
    matrix *must* be a 2d array of type float32
    char_to_index *must* be a 1d array of type int16
    rval *must* be a 1d array of type float32 and the same length as string
    """
    cdef char *buffer
    cdef Py_ssize_t len
    cdef float score
    cdef int i, j
    cdef int matrix_width = matrix.dimensions[0]
    cdef npy_int16 char_index
    # Get input string as character pointer
    if PY_MAJOR_VERSION >= 3:
        bytes_string = string.encode()
    else:
        bytes_string = string
    PyBytes_AsStringAndSize(bytes_string, &buffer, &len )
    # Loop over each position in the string 
    cdef int stop = len - matrix.dimensions[0] + 1
    for i from 0 <= i < stop:
        score = 0.0
        for j from 0 <= j < matrix_width:
            char_index = ( <npy_int16 *> ( char_to_index.data + buffer[i+j] * char_to_index.strides[0] ) )[0]
            if char_index < 0: 
                break
            score += ( <npy_float32*> ( matrix.data + j * matrix.strides[0] + char_index * matrix.strides[1] ) )[0]
        else:
            ( <npy_float32*> ( rval.data + i * rval.strides[0] ) )[0] = score
            
def score_string_with_gaps( ndarray matrix, ndarray char_to_index, object string, ndarray rval ):
    """
    Score each position in string `string` using the scoring matrix `matrix`.
    Characters in the string are mapped to columns in the matrix by `char_to_index`
    and the score for each position is stored in `rval`.

    matrix *must* be a 2d array of type float32
    char_to_index *must* be a 1d array of type int16
    rval *must* be a 1d array of type float32 and the same length as string
    """
    cdef char *buffer
    cdef Py_ssize_t len
    cdef float score
    cdef int i, j, string_pos
    cdef int matrix_width = matrix.dimensions[0]
    cdef npy_int16 char_index
    # Get input string as character pointer
    if PY_MAJOR_VERSION >= 3:
        bytes_string = string.encode()
    else:
        bytes_string = string
    PyBytes_AsStringAndSize(bytes_string, &buffer, &len )
    # Loop over each position in the string 
    cdef int stop = len - matrix.dimensions[0] + 1
    for i from 0 <= i < stop:
        if buffer[i] == '-':
            # Never start scoring at a gap
            continue
        score = 0.0
        string_pos = i
        for j from 0 <= j < matrix_width:
            # Advance to the next non-gap character
            while buffer[string_pos] == '-' and string_pos < len:
                string_pos += 1
            # Ran out of non-gap characters, no more scoring is possible
            if string_pos == len:
                return
            # Find character for position and score
            char_index = ( <npy_int16 *> ( char_to_index.data + buffer[string_pos] * char_to_index.strides[0] ) )[0]
            if char_index < 0: 
                break
            score += ( <npy_float32*> ( matrix.data + j * matrix.strides[0] + char_index * matrix.strides[1] ) )[0]
            # Matched a character, move forward
            string_pos += 1
        else:
            ( <npy_float32*> ( rval.data + i * rval.strides[0] ) )[0] = score



#!/usr/bin/env python

"""
Convert wiggle data to a binned array. This assumes the input data is on a
single chromosome and does no sanity checks!

usage: %prog score_file out_file < wiggle_data
    -c, --comp=type: compression type (none, zlib, lzo)
"""
from __future__ import division, print_function

import bx.wiggle
from bx import misc
from bx.binned_array import BinnedArray
from bx.cookbook import doc_optparse


def main():

    # Parse command line
    options, args = doc_optparse.parse(__doc__)
    try:
        if options.comp:
            comp_type = options.comp
        else:
            comp_type = None
        score_fname = args[0]
        out_fname = args[1]
    except Exception:
        doc_optparse.exit()

    scores = BinnedArray()

    for i, (chrom, pos, val) in enumerate(bx.wiggle.Reader(misc.open_compressed(score_fname))):
        # if last_chrom is None:
        #    last_chrom = chrom
        # else:
        #    assert chrom == last_chrom, "This script expects a 'wiggle' input on only one chromosome"
        scores[pos] = val
        # Status
        if i % 10000 == 0:
            print(i, "scores processed")

    out = open(out_fname, "w")
    if comp_type:
        scores.to_file(out, comp_type=comp_type)
    else:
        scores.to_file(out)
    out.close()


if __name__ == "__main__":
    main()



"""
Kanwei Li, 2009
Inspired by previous ClusterTree

Provides a ClusterTree data structure that supports efficient finding of 
clusters of intervals that are within a certain distance apart.

This clustering algorithm uses a binary tree structure. Nodes correspond to 
non-overlapping intervals, where overlapping means that the distance between
two intervals is less or equal to the max separation.

The tree self-balances using rotations based on the binomial sequence. Merges
among nodes are performed whenever a node is changed/added that will cause other
nodes to form a new cluster.

C source code is in src/cluster.c
"""

cdef extern from "cluster.h":
    
    cdef struct struct_interval:
        int start
        int end
        int id
        struct_interval * next

    ctypedef struct_interval interval
    
    cdef struct struct_clusternode:
        int start
        int end
        struct_interval *interval_head
        struct_interval *interval_tail
    
    ctypedef struct_clusternode clusternode
    
    cdef struct struct_clustertree:
        int max_dist
        int min_intervals

        struct_clusternode *root
    
    ctypedef struct_clustertree clustertree
    
    cdef struct struct_treeitr:
        struct_treeitr *next
        struct_clusternode *node
    
    ctypedef struct_treeitr treeitr
    
    clusternode* clusternode_insert(clustertree *tree, clusternode *node, int start, int end, int id)
    clustertree* create_clustertree(int max_dist, int min_intervals)
    treeitr* clusteritr(clustertree *tree)
    void freeclusteritr(treeitr *itr)
    void free_tree(clustertree *tree)

cdef class ClusterTree:
    cdef clustertree *tree
    cdef int mincols
    cdef int minregions
    
    def __cinit__(self, mincols, minregions):
        self.tree = create_clustertree(mincols, minregions)
        self.mincols = mincols
        self.minregions = minregions
        
    def __dealloc__(self):
        free_tree(self.tree)
    
    def insert(self, s, e, id):
        ''' Insert an interval with start, end, id as parameters'''
        if s > e: raise ValueError("Interval start must be before end")
        self.tree.root = clusternode_insert(self.tree, self.tree.root, s, e, id)
                
    def getregions(self):
        ''' Returns a list clusters in ascending order of starting position.
            Each cluster is a tuple of (start, end, [sorted ids of intervals in cluster])
        
        tree = ClusterTree(0, 0)
        Insert (6, 7, 1), (1, 2, 3), (9, 10, 2), (3, 4, 0), (3, 8, 4)
        tree.getregions() returns [(1, 2, [3]), (3, 8, [0, 1, 4]), (9, 10, [2])]
        '''
        cdef treeitr *itr
        cdef interval *ival
        
        regions = []
        itr = clusteritr(self.tree)
        
        while (itr):
            ids = []
            ival = itr.node.interval_head
            while (ival):
                ids.append(ival.id)
                ival = ival.next

            regions.append( (itr.node.start, itr.node.end, sorted(ids)) )
            itr = itr.next
        freeclusteritr(itr)
        return regions
        
    def getlines(self):
        ''' Similar to getregions except it just returns a list of ids of intervals
            The above example would return [3, 0, 1, 4, 2]
         '''
        cdef treeitr *itr
        cdef interval *ival
        
        lines = []
        itr = clusteritr(self.tree)
        
        while (itr):
            ids = []
            ival = itr.node.interval_head
            while (ival):
                ids.append(ival.id)
                ival = ival.next
            
            lines.extend(sorted(ids))
            itr = itr.next
        freeclusteritr(itr)
        return lines
        



"""
Tests for `bx.misc.seekbzip2`.
"""
import bz2
import os
import random
from codecs import encode

from . import seekbzip2

F = None
T = None

# F="/Users/james/work/seek-bzip2/test_random.dat.bz2"
# T="/Users/james/cache/hg18/align/multiz28way/chr10.maf.bz2"

# F=/depot/data1/cache/human/hg18/align/multiz28way/chr1.maf.bz2


if F and os.path.exists(F):

    def test_linear_reading():
        raw_data = bz2.BZ2File(F).read()
        f = seekbzip2.SeekableBzip2File(F, F + "t")
        chunk = 1221
        pos = 0
        for i in range((len(raw_data) // chunk) + 1):
            a = raw_data[pos:pos+chunk]
            b = f.read(chunk)
            assert a == b
            pos += chunk
            assert f.tell() == min(pos, len(raw_data))
        f.close()

    def test_random_seeking():
        raw_data = bz2.BZ2File(F).read()
        f = seekbzip2.SeekableBzip2File(F, F + "t")
        for i in range(10):
            seek_to = random.randrange(len(raw_data) - 100)
            chunk = random.randrange(10, 20)

            f.seek(seek_to)
            a = f.read(chunk)
            b = raw_data[seek_to: seek_to + chunk]

            assert a == b, "'%s' != '%s' on %dth attempt" % (encode(a, "hex"), encode(b, "hex"), i)

            assert f.tell() == min(seek_to + chunk, len(raw_data))
        f.close()

if T and os.path.exists(T):

    def test_text_reading():
        raw_file = bz2.BZ2File(T)
        f = seekbzip2.SeekableBzip2File(T, T + "t")
        pos = 0
        for i, (line, raw_line) in enumerate(zip(f, raw_file)):
            assert line == raw_line, "%d: %r != %r" % (i, line.rstrip(b"\n"), raw_line)
            pos += len(line)
            ftell = f.tell()
            assert ftell == pos, "%d != %d" % (ftell, pos)
        f.close()

    def test_text_reading_2():
        raw_data = bz2.BZ2File(T).read()
        f = seekbzip2.SeekableBzip2File(T, T + "t")
        raw_lines = raw_data.split(b"\n")
        pos = 0
        i = 0
        while True:
            line = f.readline()
            if line == b"":
                break
            assert line.rstrip(b"\r\n") == raw_lines[i], "%r != %r" % (line.rstrip(b"\r\n"), raw_lines[i])
            pos += len(line)
            ftell = f.tell()
            assert ftell == pos, "%d != %d" % (ftell, pos)
            i += 1
        f.close()



from __future__ import print_function

import bx.misc.bgzf


def test_bgzf():
    f = bx.misc.bgzf.BGZFFile("test_data/bgzf_tests/test.txt.gz", "r")
    assert f.read(10) == b"begin 644 "
    f.seek(0)
    assert f.read(10) == b"begin 644 "



#!/usr/bin/env python
"""
Application to convert LAV file to MAF file. Reads a LAV file from standard
input and writes a MAF file to standard out; some statistics are written to
standard error.

usage: lav_to_maf [--silent] [path=replacement] < lav_file > maf_file
"""

import sys

import bx.align.lav
import bx.align.maf


def usage(s=None):
    message = __doc__
    if s is None:
        sys.exit(message)
    else:
        sys.exit("%s\n%s" % (s, message))


def main():

    # parse the command line

    silent = False
    pathSubs = []

    for arg in sys.argv[1:]:
        if "=" in arg:
            ix = arg.find("=")
            pathSubs.append((arg[:ix], arg[ix+1:]))
        elif arg == "--silent":
            silent = True
        else:
            usage("unrecognized argument: " + arg)

    # read the alignments and other info

    out = bx.align.maf.Writer(sys.stdout)

    lavsRead = mafsWritten = 0
    for lavBlock in bx.align.lav.Reader(sys.stdin, path_subs=pathSubs):
        lavsRead += 1

        out.write(lavBlock)
        mafsWritten += 1

    if not silent:
        sys.stderr.write("%d blocks read, %d written\n" % (lavsRead, mafsWritten))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Read scores in "wiggle" format from `score_file` and intervals in "bed" format
from `interval_file` and print all scores overlapping intervals.

TODO: Support binned array format scores also.

usage: %prog score_file interval_file [out_file]
"""
from __future__ import division, print_function

import sys

import bx.wiggle
from bx import misc
from bx.binned_array import BinnedArray
from bx.cookbook import doc_optparse


def read_scores(f):
    scores_by_chrom = dict()
    for chrom, pos, val in bx.wiggle.Reader(f):
        if chrom not in scores_by_chrom:
            scores_by_chrom[chrom] = BinnedArray()
        scores_by_chrom[chrom][pos] = val
    return scores_by_chrom


def main():

    # Parse command line
    options, args = doc_optparse.parse(__doc__)
    try:
        score_file = open(args[0])
        interval_file = open(args[1])
        if len(args) > 2:
            out_file = open(args[2], 'w')
        else:
            out_file = sys.stdout
    except Exception:
        doc_optparse.exit()

    scores_by_chrom = read_scores(misc.open_compressed(sys.argv[1]))
    for line in open(sys.argv[2]):
        fields = line.split()
        chrom, start, stop = fields[0], int(fields[1]), int(fields[2])
        if chrom in scores_by_chrom:
            ba = scores_by_chrom[chrom]
            scores = [ba[i] for i in range(start, stop)]
        else:
            scores = []
        print(" ".join(fields), " ".join(map(str, scores)), file=out_file)

    score_file.close()
    interval_file.close()
    out_file.close()


if __name__ == "__main__":
    main()



#!/usr/bin/env python2.3

"""

Reads a maf file from stdin and applies the mapping file specified by
`mapping_file` to produce a sequence of integers. Then for each possible word
of length `motif_len` in this integer alphabet print the number of times
that word occurs in the block.

usage: %prog motif_len mapping_file < maf_file > counts
"""
from __future__ import division, print_function

import sys

from numpy import (
    zeros
)

import bx.align.maf
from bx import seqmapping


def main():
    word_length = int(sys.argv[1])
    with open(sys.argv[2]) as f:
        align_count, alpha_map = seqmapping.alignment_mapping_from_file(f)

    for maf in bx.align.maf.Reader(sys.stdin):
        assert len(maf.components) == align_count
        # Translate alignment to ints
        ints = seqmapping.DNA.translate_list([c.text for c in maf.components])
        # Apply mapping
        ints = alpha_map.translate(ints)
        # Count words
        radix = alpha_map.get_out_size()
        counts = zeros(radix ** word_length, int)
        total = 0
        for i in range(word_length, len(ints)):
            index = 0
            factor = 1
            skip = False
            for j in range(word_length):
                assert 0 < i-j < len(ints)
                letter = ints[i-j]
                if letter < 0:
                    skip = True
                    break
                index += letter * factor
                factor *= radix
            if skip:
                continue
            else:
                counts[index] += 1
                total += 1
        # Write ints separated by tabs
        print('\t'.join([str(total)] + [str(_) for _ in counts]))


if __name__ == "__main__":
    main()



"""
Tests for `bx.binned_array`.
"""

from numpy import (
    allclose,
    concatenate,
    NaN,
    zeros
)
from numpy.random import random_sample as random

from .binned_array import (
    BinnedArray,
    BinnedArrayWriter,
    FileBinnedArray,
)

# Bigger values take longer, but excercise more bins
CHUNK_SIZE_RANDOM = 945
CHUNK_SIZE_ZEROS = 897
# CHUNK_SIZE_RANDOM=9456
# CHUNK_SIZE_ZEROS=8972

source = target = None


def setup():
    global source
    global target
    source = []
    for i in range(13):
        if random() < 0.5:
            source = concatenate((source, random(CHUNK_SIZE_RANDOM)))
        else:
            source = concatenate((source, zeros(CHUNK_SIZE_ZEROS, 'f')))
    source = source.astype('f')
    # Set on target
    target = BinnedArray(128, NaN, len(source))
    for i in range(len(source)):
        # if not isNaN( source[i] ):
        target[i] = source[i]
    return source, target


def test_simple():
    # Verify
    for i in range(len(source)):
        assert source[i] == target[i], "No match, index: %d, source: %f, target: %f, len( source ): %d" % (i, source[i], target[i], len(source))
    # Verify with slices
    for i in range(10):
        a = int(random() * len(source))
        b = int(random() * len(source))
        if b < a:
            a, b = b, a
        assert allclose(source[a:b], target[a:b]), "No match, index: %d:%d, source: %s, target: %s" % \
            (a, b, ",".join(map(str, source[a:a+10])), ",".join(map(str, target[a:a+10])))


def test_file():
    # With a file (zlib)
    target.to_file(open("/tmp/foo", "wb"))
    target2 = FileBinnedArray(open("/tmp/foo", 'rb'))
    for i in range(len(source)):
        assert source[i] == target2[i], "No match, index: %d, source: %d, target: %d" % (i, source[i], target2[i])
    # Verify with slices
    target2 = FileBinnedArray(open("/tmp/foo", 'rb'))
    for i in range(10):
        a = int(random() * len(source))
        b = int(random() * len(source))
        if b < a:
            a, b = b, a
        assert allclose(source[a:b], target[a:b]), "No match, index: %d:%d, source: %s, target: %s" % \
            (a, b, ",".join(map(str, source[a:a+10])), ",".join(map(str, target2[a:a+10])))


def test_file_lzo():
    # With a file (lzo)
    target.to_file(open("/tmp/foo3", "wb"), comp_type="lzo")
    target3 = FileBinnedArray(open("/tmp/foo3", 'rb'))
    # Verify
    for i in range(len(source)):
        assert source[i] == target3[i], "No match, index: %d, source: %d, target: %d" % (i, source[i], target3[i])
    # Verify with slices
    target3 = FileBinnedArray(open("/tmp/foo3", 'rb'))
    for i in range(10):
        a = int(random() * len(source))
        b = int(random() * len(source))
        if b < a:
            a, b = b, a
        assert allclose(source[a:b], target3[a:b]), "No match, index: %d:%d, source: %s, target: %s" % \
            (a, b, ",".join(map(str, source[a:a+10])), ",".join(map(str, target3[a:a+10])))


def test_binned_array_writer():
    # Test with ba writer
    o = open("/tmp/foo4", "wb")
    w = BinnedArrayWriter(o, 128, comp_type='lzo')
    for val in source:
        w.write(val)
    w.finish()
    o.close()
    # Verify
    target4 = FileBinnedArray(open("/tmp/foo4", 'rb'))
    for i in range(len(source)):
        assert allclose(source[i], target4[i]), "No match, index: %d, source: %d, target: %d" % (i, source[i], target4[i])



#!/usr/bin/env python
"""
Application to convert AXT file to FASTA file. Reads an AXT file from standard
input and writes a FASTA file to standard out.

usage: %prog < axt_file > fasta_file
"""
from __future__ import print_function


__author__ = "Bob Harris (rsharris@bx.psu.edu)"

import sys

import bx.align.axt


def usage(s=None):
    message = """
axt_to_fasta < axt_file > fasta_file
"""
    if s is None:
        sys.exit(message)
    else:
        sys.exit("%s\n%s" % (s, message))


def main():
    # check the command line

    if len(sys.argv) > 1:
        usage("give me no arguments")

    # convert the alignment blocks

    reader = bx.align.axt.Reader(sys.stdin, support_ids=True,
                                 species1="", species2="")

    for a in reader:
        if "id" in a.attributes:
            id = a.attributes["id"]
        else:
            id = None
        print_component_as_fasta(a.components[0], id)
        print_component_as_fasta(a.components[1], id)
        print()


# $$$ this should be moved to a bx.align.fasta module

def print_component_as_fasta(c, id=None):
    header = ">%s_%s_%s" % (c.src, c.start, c.start + c.size)
    if id is not None:
        header += " " + id
    print(header)
    print(c.text)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Read a wiggle track and print out a series of lines containing
"chrom position score". Ignores track lines, handles bed, variableStep
and fixedStep wiggle lines.
"""
from __future__ import print_function

import sys

import bx.wiggle

if len(sys.argv) > 1:
    in_file = open(sys.argv[1])
else:
    in_file = sys.stdin

if len(sys.argv) > 2:
    out_file = open(sys.argv[2], "w")
else:
    out_file = sys.stdout

for fields in bx.wiggle.Reader(in_file):
    print(" ".join(map(str, fields)))

in_file.close()
out_file.close()



"""
Compact mutable sequences of bits (vectors of 0s and 1s) supporting various
boolean operations, and a "binned" variation which stores long runs of 
identical bits compactly.

Because the binned implementation avoids a lot of memory allocation and access
when working with either small subregions of the total interval or setting /
testing spans larger than the bin size, it can be much faster.
"""

import sys

cdef extern from "common.h":
    ctypedef int boolean

cdef extern from "bits.h":
    ctypedef unsigned char Bits
    # Allocate bits. 
    Bits * bitAlloc( int bitCount )
    # Clone bits. 
    Bits * bitClone(Bits* orig, int bitCount )
    # Free bits. 
    void bitFree(Bits **pB)
    # Set a single bit. 
    void bitSetOne(Bits *b, int bitIx)
    # Clear a single bit. 
    void bitClearOne(Bits *b, int bitIx)
    # Set a range of bits. 
    void bitSetRange(Bits *b, int startIx, int bitCount)
    # Read a single bit. 
    int bitReadOne(Bits *b, int bitIx)
    # Count number of bits set in range. 
    int bitCountRange(Bits *b, int startIx, int bitCount)
    # Find the index of the the next set bit. 
    int bitFindSet(Bits *b, int startIx, int bitCount)
    # Find the index of the the next clear bit. 
    int bitFindClear(Bits *b, int startIx, int bitCount)
    # Clear many bits. 
    void bitClear(Bits *b, int bitCount)
    # And two bitmaps.  Put result in a. 
    void bitAnd(Bits *a, Bits *b, int bitCount)
    # Or two bitmaps.  Put result in a. 
    void bitOr(Bits *a, Bits *b, int bitCount)
    # Xor two bitmaps.  Put result in a. 
    void bitXor(Bits *a, Bits *b, int bitCount)
    # Flip all bits in a. 
    void bitNot(Bits *a, int bitCount)
    ## # Print part or all of bit map as a string of 0s and 1s.  Mostly useful for
    ## void bitPrint(Bits *a, int startIx, int bitCount, FILE* out)

cdef extern from "binBits.h":
    struct BinBits:
        int size
        int bin_size
        int nbins
        Bits ** bins
    BinBits* binBitsAlloc( int size, int granularity )
    void binBitsFree( BinBits * bb )
    int binBitsReadOne( BinBits * bb, int pos )
    void binBitsSetOne( BinBits * bb, int pos )
    void binBitsClearOne( BinBits * bb, int pos )
    void binBitsSetRange( BinBits *bb, int start, int size )
    int binBitsCountRange( BinBits *bb, int start, int size )
    int binBitsFindSet( BinBits *bb, int start )
    int binBitsFindClear( BinBits *bb, int start )
    void binBitsAnd( BinBits *bb1, BinBits *bb2 )
    void binBitsOr( BinBits *bb1, BinBits *bb2 )
    void binBitsNot( BinBits *bb )

## ---- Forward declerations ------------------------------------------------

cdef class BitSet
cdef class BinnedBitSet

## ---- BitSet bounds checking ----------------------------------------------
    
cdef inline b_check_index( BitSet b, index ):
    if index < 0:
        raise IndexError( "BitSet index (%d) must be non-negative." % index )
    if index >= b.bitCount:
        raise IndexError( "%d is larger than the size of this BitSet (%d)." % ( index, b.bitCount ) )
    
cdef inline b_check_range( BitSet b, start, end ):
    b_check_index( b, start )
    if end < start:
        raise IndexError( "Range end (%d) must be greater than range start(%d)." % ( end, start ) )
    if end > b.bitCount:
        raise IndexError( "End %d is larger than the size of this BitSet (%d)." % ( end, b.bitCount ) )
        
cdef inline b_check_range_count( BitSet b, start, count ):
    b_check_index( b, start )
    if count < 0:
        raise IndexError( "Count (%d) must be non-negative." % count )
    if start + count > b.bitCount:
        raise IndexError( "End %d is larger than the size of this BitSet (%d)." % ( start + count, b.bitCount ) )

cdef inline b_check_same_size( BitSet b, BitSet other ):
    if b.bitCount != other.bitCount:
        raise ValueError( "BitSets must have the same size" )

## ---- BitSet --------------------------------------------------------------

# Maximum value of a signed 32 bit integer ( 2**31 - 1 )
cdef int MAX_INT = 2147483647

cdef class BitSet:
    cdef Bits * bits
    cdef int bitCount
    def __cinit__( self, bitCount ):
        if bitCount > MAX_INT:
            raise ValueError( "%d is larger than the maximum BitSet size of %d." % ( bitCount, MAX_INT ) )
        self.bitCount = bitCount
        self.bits = bitAlloc( bitCount )
    def __dealloc__( self ):
        if self.bits:
            bitFree( & self.bits )
    property size:
        def __get__( self ):
            return self.bitCount
    def set( self, index ):
        b_check_index( self, index )
        bitSetOne( self.bits, index )
    def clear( self, index ):
        b_check_index( self, index )
        bitClearOne( self.bits, index )
    def clone( self ):
        other = BitSet( self.bitCount )
        other.ior( self )
        return other
    def set_range( self, start, count ):
        b_check_range_count( self, start, count )
        bitSetRange( self.bits, start, count )
    def get( self, index ):
        b_check_index( self, index )
        return bitReadOne( self.bits, index );
    def count_range( self, start=0, count=None ):
        if count == None: 
            count = self.bitCount - start
        b_check_range_count( self, start, count )
        return bitCountRange( self.bits, start, count )
    def next_set( self, start, end=None ):
        if end == None: 
            end = self.bitCount
        b_check_range( self, start, end )
        return bitFindSet( self.bits, start, end )
    def next_clear( self, start, end=None ):
        if end == None: 
            end = self.bitCount
        b_check_range( self, start, end )
        return bitFindClear( self.bits, start, end )
    def iand( self, BitSet other ):
        b_check_same_size( self, other )
        bitAnd( self.bits, other.bits, self.bitCount )
    def ior( self, BitSet other ): 
        b_check_same_size( self, other )
        bitOr( self.bits, other.bits, self.bitCount )
    def ixor( self, BitSet other ): 
        b_check_same_size( self, other )
        bitXor( self.bits, other.bits, self.bitCount )
    def invert( self ):
        bitNot( self.bits, self.bitCount)
    def __getitem__( self, index ):
        return self.get( index )
    def __iand__( self, other ):
        self.iand( other )
        return self
    def __ior__( self, other ):
        self.ior( other )
        return self
    def __invert__( self ):
        self.invert()
        return self

## ---- BinnedBitSet bounds checking ----------------------------------------

cdef inline bb_check_index( BinnedBitSet bb, index ):
    if index < 0:
        raise IndexError( "BitSet index (%d) must be non-negative." % index )
    if index >= bb.bb.size:
        raise IndexError( "%d is larger than the size of this BitSet (%d)." % ( index, bb.bb.size ) )        
cdef inline bb_check_start( BinnedBitSet bb, start ):
    bb_check_index( bb, start )
cdef inline bb_check_range_count( BinnedBitSet bb, start, count ):
    bb_check_index( bb, start )
    if count < 0:
        raise IndexError( "Count (%d) must be non-negative." % count )
    if start + count > bb.bb.size:
        raise IndexError( "End (%d) is larger than the size of this BinnedBitSet (%d)." % ( start + count, bb.bb.size ) )
cdef inline bb_check_same_size( BinnedBitSet bb, BinnedBitSet other ):
    if bb.bb.size != other.bb.size:
        raise ValueError( "BitSets must have the same size" )

## ---- BinnedBitSet --------------------------------------------------------

MAX=512*1024*1024 

cdef class BinnedBitSet:
    cdef BinBits * bb
    def __cinit__( self, size=MAX, granularity=1024 ):
        if size > MAX_INT:
            raise ValueError( "%d is larger than the maximum BinnedBitSet size of %d." % ( size, MAX_INT ) )
        self.bb = binBitsAlloc( size, granularity )
    def __dealloc__( self ):
        if self.bb:
            binBitsFree( self.bb );
    def __getitem__( self, index ):
        bb_check_index( self, index )
        return binBitsReadOne( self.bb, index )
    def set( self, index ):
        bb_check_index( self, index )
        binBitsSetOne( self.bb, index )
    def clear( self, index ):
        bb_check_index( self, index )
        binBitsClearOne( self.bb, index )
    def set_range( self, int start, count ):
        bb_check_range_count( self, start, count )
        binBitsSetRange( self.bb, start, count )
    def count_range( self, start, count ):
        bb_check_range_count( self, start, count )
        return binBitsCountRange( self.bb, start, count )
    def next_set( self, start ):
        bb_check_start( self, start )
        return binBitsFindSet( self.bb, start )
    def next_clear( self, start ):
        bb_check_start( self, start )
        return binBitsFindClear( self.bb, start )
    property size:
        def __get__( self ):
            return self.bb.size
    property bin_size:
        def __get__( self ):
            return self.bb.bin_size
    def iand( self, BinnedBitSet other ):
        bb_check_same_size( self, other )
        binBitsAnd( self.bb, other.bb )
    def ior( self, BinnedBitSet other ):
        bb_check_same_size( self, other )
        binBitsOr( self.bb, other.bb )
    def invert( self ):
        binBitsNot( self.bb )



#!/usr/bin/env python2.4

"""
Match up intersecting intervals from two files. This performs a "full join",
any pair of intervals with any basewise overlap will be printed side-by-side.

usage: %prog bed1 bed2
"""
from __future__ import division, print_function

import sys

import bx.intervals.intersection
import bx.intervals.io


def main():
    intersecters = {}

    # Read second set into intersecter
    for interval in bx.intervals.io.GenomicIntervalReader(open(sys.argv[2])):
        if interval.chrom not in intersecters:
            intersecters[interval.chrom] = bx.intervals.Intersecter()
        intersecters[interval.chrom].add_interval(interval)

    # Join with first set
    for interval in bx.intervals.io.GenomicIntervalReader(open(sys.argv[1])):
        if interval.chrom in intersecters:
            intersection = intersecters[interval.chrom].find(interval.start, interval.end)
            for interval2 in intersection:
                print("\t".join([str(interval), str(interval2)]))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Application to convert AXT file to MAF file. Reads an AXT file from standard
input and writes a MAF file to standard out;  some statistics are written to
standard error.

axt_to_maf primary:lengths_file secondary:lengths_file < axt_file > maf_file
  --silent: prevents stats report

  Lengths files provide the length of each chromosome (maf format needs this
  information but axt file does not contain it).  The format is a series of
  lines of the form:

    <chromosome name> <length>

  The chromosome field in each axt block must match some <chromosome name> in
  the lengths file.
"""

__author__ = "Bob Harris (rsharris@bx.psu.edu)"

import copy
import sys

import bx.align.axt
import bx.align.maf


def usage(s=None):
    message = __doc__
    if (s is None):
        sys.exit(message)
    else:
        sys.exit("%s\n%s" % (s, message))


def main():
    global debug

    ##########
    # parse the command line
    ##########

    primary = None
    secondary = None
    silent = False

    # pick off options

    args = sys.argv[1:]
    while (len(args) > 0):
        arg = args.pop(0)
        val = None
        fields = arg.split("=", 1)
        if (len(fields) == 2):
            arg = fields[0]
            val = fields[1]
            if (val == ""):
                usage("missing a value in %s=" % arg)

        if (arg == "--silent") and (val is None):
            silent = True
        elif (primary is None) and (val is None):
            primary = arg
        elif (secondary is None) and (val is None):
            secondary = arg
        else:
            usage("unknown argument: %s" % arg)

    if (primary is None):
        usage("missing primary species")

    if (secondary is None):
        usage("missing secondary species")

    fields = primary.split(":")
    if (len(fields) != 2):
        usage("bad primary species (must be species:lengths_file")
    primary = fields[0]
    primaryLengths = fields[1]

    fields = secondary.split(":")
    if (len(fields) != 2):
        usage("bad secondary species (must be species:lengths_file")
    secondary = fields[0]
    secondaryLengths = fields[1]

    ##########
    # read the lengths
    ##########

    speciesToLengths = {}
    speciesToLengths[primary] = read_lengths(primaryLengths)
    speciesToLengths[secondary] = read_lengths(secondaryLengths)

    ##########
    # read the alignments
    ##########

    out = bx.align.maf.Writer(sys.stdout)

    axtsRead = 0
    axtsWritten = 0
    for axtBlock in bx.align.axt.Reader(
            sys.stdin,
            species_to_lengths=speciesToLengths,
            species1=primary,
            species2=secondary):
        axtsRead += 1

        p = axtBlock.get_component_by_src_start(primary)
        if (p is None):
            continue
        s = axtBlock.get_component_by_src_start(secondary)
        if (s is None):
            continue

        mafBlock = bx.align.Alignment(axtBlock.score, axtBlock.attributes)
        mafBlock.add_component(clone_component(p))
        mafBlock.add_component(clone_component(s))

        out.write(mafBlock)
        axtsWritten += 1

    if (not silent):
        sys.stderr.write("%d blocks read, %d written\n" % (axtsRead, axtsWritten))


def clone_component(c):
    return bx.align.Component(c.src, c.start, c.size, c.strand, c.src_size, copy.copy(c.text))


def read_lengths(fileName):

    chromToLength = {}

    f = open(fileName, "r")

    for lineNumber, line in enumerate(f):
        line = line.strip()
        if (line == ""):
            continue
        if (line.startswith("#")):
            continue

        fields = line.split()
        if (len(fields) != 2):
            raise ValueError("bad lengths line (%s:%d): %s" % (fileName, lineNumber, line))

        chrom = fields[0]
        try:
            length = int(fields[1])
        except ValueError:
            raise ValueError("bad lengths line (%s:%d): %s" % (fileName, lineNumber, line))

        if (chrom in chromToLength):
            raise ValueError("%s appears more than once (%s:%d): %s" % (chrom, fileName, lineNumber, line))

        chromToLength[chrom] = length

    f.close()

    return chromToLength


if __name__ == "__main__":
    main()



"""
Seekable access to BGZ files based on samtools code. Does not yet implement
complete file-like interface.
"""

from cpython.version cimport PY_MAJOR_VERSION

ctypedef unsigned long long int64_t

cdef extern from "Python.h":
    char * PyBytes_AsString( object )
    object PyBytes_FromStringAndSize( char *, Py_ssize_t )

cdef extern from "bgzf.h":
    ctypedef struct BGZF
    BGZF * bgzf_open( const char * path, const char * mode )
    int bgzf_close( BGZF * fp )
    int bgzf_read( BGZF * fp, void * data, int length )
    int64_t bgzf_tell( BGZF * fp )
    int64_t bgzf_seek( BGZF * fp, int64_t pos, int where )
    
cdef class BGZFFile( object ):
    cdef BGZF * bgzf
    def __init__( self, path, mode="r" ):
        if PY_MAJOR_VERSION >= 3:
            bytes_path, bytes_mode = path.encode(), mode.encode()
        else:
            bytes_path, bytes_mode = path, mode
        self.bgzf = bgzf_open( bytes_path, bytes_mode )
        if not self.bgzf:
            raise IOError( "Could not open file" )
      
    def close( self ):
        if self.bgzf:
            bgzf_close( self.bgzf )
    def read( self, int length ):
        cdef object rval
        rval = PyBytes_FromStringAndSize( NULL, length )
        bgzf_read( self.bgzf, PyBytes_AsString( rval ), length )
        return rval
    def tell( self ):
        return bgzf_tell( self.bgzf )
    def seek( self, int64_t pos, int where=0 ):
        return bgzf_seek( self.bgzf, pos, where )



#!/usr/bin/env python

"""
'Tile' the blocks of a maf file over each of a set of intervals. The
highest scoring block that covers any part of a region will be used, and
pieces not covered by any block filled with "-" or optionally "*".

This version uses synteny annotation if found on the alignment blocks, and
will attempt to fill gaps with special characters depending on the type of
gap, similar to the projected alignment display of the UCSC genome browser:
'*' for new, '=' for inverse/inset, '#' for contig, 'X' for missing.

- The list of species to tile is specified by the first argument (either a
  newick tree or just a comma separated list).

- a 2bit file is expected for the reference species to fill in missing
  sequence

- The remaining arguments are a list of maf files which must have
  corresponding ".index" files.

TODO: The seq_db format is specific to something old and obsure at PSU,
      need to standardize.

usage: %prog list,of,species,to,keep ref.2bit indexed_maf_files ...
    -m, --missingData: Inserts wildcards for missing block rows instead of '-'
    -s, --strand:      Use strand information for intervals, reveres complement if '-'
"""

import string
import sys

import bx.align as align
import bx.align.maf as maf
import bx.seq.nib
import bx.seq.twobit
from bx.cookbook import doc_optparse

tree_tx = string.maketrans("(),", "   ")


def main():

    options, args = doc_optparse.parse(__doc__)
    try:
        sources = args[0].translate(tree_tx).split()
        ref_2bit = bx.seq.twobit.TwoBitFile(open(args[1]))
        index = maf.MultiIndexed(args[2:])

        out = maf.Writer(sys.stdout)
        missing_data = bool(options.missingData)
        use_strand = bool(options.strand)
    except Exception:
        doc_optparse.exception()

    for line in sys.stdin:
        fields = line.split()
        ref_src, start, end = fields[0:3]
        if use_strand and len(fields) > 5:
            strand = fields[5]
        else:
            strand = '+'
        do_interval(sources, index, out, ref_src, int(start), int(end), ref_2bit, missing_data, strand)

    out.close()


def get_fill_char(maf_status):
    """
    Return the character that should be used to fill between blocks
    having a given status
    """
    # assert maf_status not in (maf.MAF_CONTIG_NESTED_STATUS, maf.MAF_NEW_NESTED_STATUS,
    #                           maf.MAF_MAYBE_NEW_NESTED_STATUS ), \
    #     "Nested rows do not make sense in a single coverage MAF (or do they?)"
    if maf_status in (maf.MAF_NEW_STATUS, maf.MAF_MAYBE_NEW_STATUS,
                      maf.MAF_NEW_NESTED_STATUS, maf.MAF_MAYBE_NEW_NESTED_STATUS):
        return "*"
    elif maf_status in (maf.MAF_INVERSE_STATUS, maf.MAF_INSERT_STATUS):
        return "="
    elif maf_status in (maf.MAF_CONTIG_STATUS, maf.MAF_CONTIG_NESTED_STATUS):
        return "#"
    elif maf_status == maf.MAF_MISSING_STATUS:
        return "X"
    else:
        raise ValueError("Unknwon maf status")


def guess_fill_char(left_comp, right_comp):
    """
    For the case where there is no annotated synteny we will try to guess it
    """
    # No left component, obiously new
    return "*"
    # First check that the blocks have the same src (not just species) and
    # orientation
    if left_comp.src == right_comp.src and left_comp.strand != right_comp.strand:
        # Are they completely contiguous? Easy to call that a gap
        if left_comp.end == right_comp.start:
            return "-"
        # TODO: should be able to make some guesses about short insertions
        # here
    # All other cases we have no clue about
    return "*"


def remove_all_gap_columns(texts):
    """
    Remove any columns containing only gaps from alignment texts
    """
    seqs = [list(t) for t in texts]
    i = 0
    text_size = len(texts[0])
    while i < text_size:
        all_gap = True
        for seq in seqs:
            if seq[i] not in ('-', '#', '*', '=', 'X', '@'):
                all_gap = False
        if all_gap:
            for seq in seqs:
                del seq[i]
            text_size -= 1
        else:
            i += 1
    return [''.join(s) for s in seqs]


def do_interval(sources, index, out, ref_src, start, end, ref_2bit, missing_data, strand):
    """
    Join together alignment blocks to create a semi human projected local
    alignment (small reference sequence deletions are kept as supported by
    the local alignment).
    """
    ref_src_size = None
    # Make sure the reference component is also the first in the source list
    assert sources[0].split('.')[0] == ref_src.split('.')[0], "%s != %s" \
        % (sources[0].split('.')[0], ref_src.split('.')[0])
    # Extract non-species part from ref_src for grabbing sequence
    ref_chr = ref_src
    if "." in ref_src:
        ref_chr = ref_src[ref_src.index(".")+1:]
    # Counter for the last reference species base we have processed
    last_stop = start
    # Rows in maf blocks come in in arbitrary order, we'll convert things
    # to the destred order of the tiled block
    source_to_index = dict((name, i) for (i, name) in enumerate(sources))
    # This gets all the maf blocks overlapping our interval of interest
    # NOTE: Unlike maf_tile we're expecting
    # things to be single coverage in the reference species, so we won't
    # sort by score and lay down.
    blocks = index.get(ref_src, start, end)
    # The last component seen for each species onto which we are tiling
    last_components = [None] * len(sources)
    last_status = [None] * len(sources)
    cols_needing_fill = [0] * len(sources)
    # The list of strings in which we build up the tiled alignment
    tiled_rows = ["" for i in range(len(sources))]
    # Enumerate the (ordered) list of blocks
    for i, block in enumerate(blocks):
        # Check for overlap in reference species
        ref = block.get_component_by_src_start(ref_src)
        if ref.start < last_stop:
            if ref.end < last_stop:
                continue
            block = block.slice_by_component(ref, last_stop, min(end, ref.end))
            ref = block.get_component_by_src_start(ref_src)
        block = block.slice_by_component(ref, max(start, ref.start), min(end, ref.end))
        ref = block.get_component_by_src_start(ref_src)
        # print block
        assert last_components[0] is None or ref.start >= last_components[0].end, \
            "MAF must be sorted and single coverage in reference species!"
        assert ref.strand == "+", \
            "MAF must have all reference species blocks on the plus strand"
        # Store the size of the reference sequence for building fake block
        if ref_src_size is None:
            ref_src_size = ref.src_size
        # Handle the reference component seperately, it has no synteny status
        # but we will try to fill in missing sequence
        if ref.start > last_stop:
            # Need to fill in some reference sequence
            chunk_len = ref.start - last_stop
            text = ref_2bit[ref_chr].get(last_stop, last_stop + chunk_len)
            tiled_rows[0] += text
            for source in sources[1:]:
                cols_needing_fill[source_to_index[source]] += chunk_len
        # Do reference component
        chunk_len = len(ref.text)
        tiled_rows[0] += ref.text
        # Do each other component
        for source in sources[1:]:
            source_index = source_to_index[source]
            comp = block.get_component_by_src_start(source)
            if comp:
                if comp.synteny_left is None:
                    left_status, left_length = None, -1
                else:
                    left_status, left_length = comp.synteny_left
                if comp.synteny_right is None:
                    right_status, right_length = None, -1
                else:
                    right_status, right_length = comp.synteny_right
                # We have a component, do we need to do some filling?
                cols_to_fill = cols_needing_fill[source_index]
                if cols_to_fill > 0:
                    # Adjacent components should have matching status
                    # assert last_status[ source_index ] is None or last_status[ source_index ] == left_status, \
                    #     "left status (%s) does not match right status (%s) of last component for %s" \
                    #     % ( left_status, last_status[ source_index ], source )
                    if left_status is None:
                        fill_char = guess_fill_char(last_components[source_index], comp)
                    else:
                        fill_char = get_fill_char(left_status)
                    tiled_rows[source_index] += (fill_char * cols_to_fill)
                    cols_needing_fill[source_index] = 0
                # Okay, filled up to current position, now append the text
                tiled_rows[source_index] += comp.text
                assert len(tiled_rows[source_index]) == len(tiled_rows[0]), \
                    "length of tiled row should match reference row"
                last_components[source_index] = comp
                last_status[source_index] = right_status
            else:
                # No component, we'll have to fill this region when we know
                # the status
                cols_needing_fill[source_index] += chunk_len
        last_stop = ref.end
    # No more components, clean up the ends
    if last_stop < end:
        # Need to fill in some reference sequence
        chunk_len = end - last_stop
        tiled_rows[0] += ref_2bit[ref_chr].get(last_stop, last_stop + chunk_len)
        for source in sources[1:]:
            cols_needing_fill[source_to_index[source]] += chunk_len
    # Any final filling that needs to be done?
    for source in sources[1:]:
        source_index = source_to_index[source]
        fill_needed = cols_needing_fill[source_index]
        if fill_needed > 0:
            if last_components[source_index] is None:
                # print >>sys.stderr, "Never saw any components for %s, filling with @" % source
                fill_char = '@'
            else:
                if last_status[source_index] is None:
                    fill_char = '*'
                else:
                    fill_char = get_fill_char(last_status[source_index])
            tiled_rows[source_index] += fill_char * fill_needed
        assert len(tiled_rows[source_index]) == len(tiled_rows[0]), \
            "length of tiled row should match reference row"
    # Okay, now make up the fake alignment from the tiled rows.
    tiled_rows = remove_all_gap_columns(tiled_rows)
    a = align.Alignment()
    for i, name in enumerate(sources):
        text = "".join(tiled_rows[i])
        size = len(text) - text.count("-")
        if i == 0:
            if ref_src_size is None:
                ref_src_size = ref_2bit[ref_chr].length
            c = align.Component(ref_src, start, end-start, "+", ref_src_size, text)
        else:
            c = align.Component(name + ".fake", 0, size, "?", size, text)
        a.add_component(c)
    if strand == '-':
        a = a.reverse_complement()
    out.write(a)


main()



#!/usr/bin/env python
"""
Application to convert MAF file to AXT file, projecting to any two species.
Reads a MAF file from standard input and writes an AXT file to standard out;
some statistics are written to standard error.  The user must specify the
two species of interest.

usage: %prog primary_species secondary_species < maf_file > axt_file
"""

__author__ = "Bob Harris (rsharris@bx.psu.edu)"

import copy
import sys

import bx.align.axt
import bx.align.maf


def usage(s=None):
    message = """
maf_to_axt primary_species secondary_species < maf_file > axt_file
"""
    if s is None:
        sys.exit(message)
    else:
        sys.exit("%s\n%s" % (s, message))


def main():
    primary = None
    secondary = None

    args = sys.argv[1:]
    while len(args) > 0:
        arg = args.pop(0)
        val = None
        fields = arg.split("=", 1)
        if len(fields) == 2:
            arg = fields[0]
            val = fields[1]
            if val == "":
                usage("missing a value in %s=" % arg)

        if primary is None and val is None:
            primary = arg
        elif secondary is None and val is None:
            secondary = arg
        else:
            usage("unknown argument: %s" % arg)

    if primary is None:
        usage("missing primary species")

    if secondary is None:
        usage("missing secondary species")

    # read the alignments and other info

    out = bx.align.axt.Writer(sys.stdout)

    axtsRead = 0
    mafsWritten = 0
    for mafBlock in bx.align.maf.Reader(sys.stdin):
        axtsRead += 1

        p = mafBlock.get_component_by_src_start(primary)
        if p is None:
            continue
        s = mafBlock.get_component_by_src_start(secondary)
        if s is None:
            continue

        axtBlock = bx.align.Alignment(mafBlock.score, mafBlock.attributes)
        axtBlock.add_component(clone_component(p))
        axtBlock.add_component(clone_component(s))

        remove_mutual_gaps(axtBlock)
        if axtBlock.text_size == 0:
            continue

        out.write(axtBlock)
        mafsWritten += 1

    sys.stderr.write("%d blocks read, %d written\n" % (axtsRead, mafsWritten))


def clone_component(c):
    return bx.align.Component(c.src, c.start, c.size, c.strand, c.src_size, copy.copy(c.text))


def remove_mutual_gaps(block):

    if len(block.components) == 0:
        return

    nonGaps = []

    for c in block.components:
        for ix in range(0, block.text_size):
            if ix not in nonGaps and c.text[ix] != "-":
                nonGaps.append(ix)

    nonGaps.sort()

    for c in block.components:
        c.text = "".join([c.text[ix] for ix in nonGaps])

    block.text_size = len(nonGaps)


if __name__ == "__main__":
    main()



"""
Support for scores in the `wiggle`_ file format used by the UCSC Genome 
Browser.

The positions in the wiggle format are 1-relative, however,
the positions returned match the BED/interval format which is zero-based, half-open.

.. _wiggle: http://genome.ucsc.edu/goldenPath/help/wiggle.html
"""

def parse_header( line ):
    return dict( [ field.split( '=' ) for field in line.split()[1:] ] )

cdef class WiggleReader:
    """
    Iterator yielding chrom, start, end, strand, value.
    Values are zero-based, half-open.
    Regions which lack a score are ignored.
    """
    #cdef object file
    #cdef object current_chrom
    #cdef long current_pos
    #cdef long current_step
    #cdef long current_span
    #cdef linemode mode
    def __init__( self, file ):
        self.file = file
        self.current_chrom = None
        self.current_pos = -1
        self.current_step = -1
        self.current_span = -1
        self.mode = MODE_BED

    def __iter__( self ):
        return self

    def __next__( self ):
        while True:
            line = self.file.readline()
            if not line:
                raise StopIteration()
            if line.isspace():
                continue    
            if line[0] == "#":
                continue
            if line[0].isalpha():
                if line.startswith( "track" ) or line.startswith( "browser" ):
                    continue
                elif line.startswith( "variableStep" ):
                    header = parse_header( line )
                    self.current_chrom = header['chrom']
                    self.current_pos = -1
                    self.current_step = -1
                    if 'span' in header:
                        self.current_span = int( header['span'] )
                    else:
                        self.current_span = 1
                    self.mode = MODE_VARIABLE
                    continue
                elif line.startswith( "fixedStep" ):
                    header = parse_header( line )
                    self.current_chrom = header['chrom']
                    self.current_pos = int( header['start'] ) - 1
                    self.current_step = int( header['step'] )
                    if 'span' in header:
                        self.current_span = int( header['span'] )
                    else:
                        self.current_span = 1
                    self.mode = MODE_FIXED
                    continue
            elif self.mode == MODE_BED:
                fields = line.split()
                if len( fields ) > 3:
                    if len( fields ) > 5:
                        return fields[0], int( fields[1] ), int( fields[2] ), fields[5], float( fields[3] )
                    else:
                        return fields[0], int( fields[1] ), int( fields[2] ), "+", float( fields[3] )
            elif self.mode == MODE_VARIABLE: 
                fields = line.split()
                try:
                    pos = int( fields[0] ) - 1
                    val = float( fields[1] )
                except ValueError:
                    continue
                return self.current_chrom, pos, pos + self.current_span, "+", val
            elif self.mode == MODE_FIXED:
                fields = line.split()
                try:
                    val = float( fields[0] )
                except ValueError:
                    continue
                return self.current_chrom, self.current_pos, self.current_pos + self.current_span, "+", val
                # FIXME: unreachable! need to test this and fix!
                self.current_pos += self.current_step
            else:
                raise "Unexpected input line: %s" % line.strip()





"""
Classes for implementing `Pipeline`s composed of `Filter`s (intended to be
subclassed).
"""


class Filter(object):
    def __init__(self, **kwargs):
        raise Exception("AbstractClass")

    def run(self, reader, writer):
        for block in reader:
            block = self(block)
            if block:
                writer(block)

    def step(self, reader, writer):
        block = next(reader)
        if not block:
            raise StopIteration
        block = self(block)
        if block:
            writer(block)

    def __call__(self, block):
        raise Exception("AbstractMethod")


class Pipeline(Filter):
    def __init__(self, **kwargs):
        self.pipeline = list()

    def __call__(self, block):
        for function in self.pipeline:
            if not block:
                return block
            try:
                f = function.__call__
            except AttributeError:
                raise TypeError("'" + function.__class__.__name__ + "' is not callable.")
            block = f(block)
        return block

    def append(self, function):
        try:
            function.__call__
        except AttributeError:
            raise TypeError("'" + function.__class__.__name__ + "' is not callable.")
        return self.pipeline.append(function)

    def remove(self, function):
        return self.pipeline.remove(function)

    def extend(self, pipeline):
        for item in pipeline:
            self.append(item)

    # Container interface
    def __len__(self):
        return len(self.pipeline)

    def __getitem__(self, key):
        return self.pipeline[key]

    def __setitem__(self, key, value):
        try:
            value.__call__
        except AttributeError:
            raise TypeError("'" + value.__class__.__name__ + "' is not callable.")
        return self.pipeline.__setitem__(key, value)

    def __delitem__(self, key):
        return self.pipeline.__delitem__(key)

    def __iter__(self):
        return self.pipeline.__iter__()

    def __contains__(self, item):
        return self.pipeline.__contains__(item)



#!/usr/bin/env python

"""
Read data in UCSC wiggle format and write it to an "array tree" file.

usage: %prog array_length output.array_tree < input.wig
"""

from __future__ import division

import sys

from bx.arrays.array_tree import (
    array_tree_dict_from_reader,
    FileArrayTreeDict
)
from bx.arrays.wiggle import WiggleReader


def main():

    sizes_fname = sys.argv[1]
    out_fname = sys.argv[2]

    sizes = {}
    for line in open(sizes_fname):
        fields = line.split()
        sizes[fields[0]] = int(fields[1])

    # Fill array from wiggle
    d = array_tree_dict_from_reader(WiggleReader(sys.stdin), sizes)

    for value in d.values():
        value.root.build_summary()

    with open(out_fname, "w") as f:
        FileArrayTreeDict.dict_to_file(d, f)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
For every column that occurs in a multiple alignment print the column
and the number of times it occurs (one column/count per line, tab
separated), sorted by count descending.

Note: all blocks must have exactly the same number of species.

usage: %prog < maf > column_counts
"""
from __future__ import print_function

import sys

import bx.align.maf

counts = {}

nspecies = None

for block in bx.align.maf.Reader(sys.stdin):
    # Ensure all blocks have the same number of rows
    if nspecies:
        assert len(block.components) == nspecies
    else:
        nspecies = len(block.components)
    # Increment count for each column
    for col in zip(* [iter(comp.text.upper()) for comp in block.components]):
        try:
            counts[col] += 1
        except Exception:
            counts[col] = 1

counts = sorted((value, key) for key, value in counts.items())
counts.reverse()

for count, col in counts:
    print("".join(col), count)



from __future__ import division

from io import BytesIO

from six import Iterator

from bx_extras.lrucache import LRUCache

DEFAULT_CACHE_SIZE = 10
DEFAULT_BLOCK_SIZE = 1024*1024*2


class FileCache(Iterator):
    """
    Wrapper for a file that cache blocks of data in memory.

    **NOTE:** this is currently an incomplete file-like object, it only
    supports seek, tell, and readline (plus iteration). Reading bytes is
    currently not implemented.
    """

    def __init__(self, file, size, cache_size=DEFAULT_CACHE_SIZE, block_size=DEFAULT_BLOCK_SIZE):
        """
        Create a new `FileCache` wrapping the file-like object `file` that
        has total size `size` and caching blocks of size `block_size`.
        """
        self.file = file
        self.size = size
        self.cache_size = cache_size
        self.block_size = block_size
        # Setup the cache
        self.nblocks = (self.size // self.block_size) + 1
        self.cache = LRUCache(self.cache_size)
        # Position in file
        self.dirty = True
        self.at_eof = False
        self.file_pos = 0
        self.current_block_index = -1
        self.current_block = None

    def fix_dirty(self):
        chunk, offset = self.get_block_and_offset(self.file_pos)
        if self.current_block_index != chunk:
            self.current_block = BytesIO(self.load_block(chunk))
            self.current_block.read(offset)
            self.current_block_index = chunk
        else:
            self.current_block.seek(offset)
        self.dirty = False

    def get_block_and_offset(self, index):
        return int(index // self.block_size), int(index % self.block_size)

    def load_block(self, index):
        if index in self.cache:
            return self.cache[index]
        else:
            real_offset = index * self.block_size
            self.file.seek(real_offset)
            block = self.file.read(self.block_size)
            self.cache[index] = block
            return block

    def seek(self, offset, whence=0):
        """
        Move the file pointer to a particular offset.
        """
        # Determine absolute target position
        if whence == 0:
            target_pos = offset
        elif whence == 1:
            target_pos = self.file_pos + offset
        elif whence == 2:
            target_pos = self.size - offset
        else:
            raise Exception("Invalid `whence` argument: %r", whence)
        # Check if this is a noop
        if target_pos == self.file_pos:
            return
        # Verify it is valid
        assert 0 <= target_pos < self.size, "Attempt to seek outside file"
        # Move the position
        self.file_pos = target_pos
        # Mark as dirty, the next time a read is done we need to actually
        # move the position in the bzip2 file
        self.dirty = True

    def readline(self):
        if self.dirty:
            self.fix_dirty()
        if self.at_eof:
            return b""
        rval = []
        while True:
            line = self.current_block.readline()
            rval.append(line)
            if len(line) > 0 and line[-1] == b'\n':
                break
            elif self.current_block_index == self.nblocks - 1:
                self.at_eof = True
                break
            else:
                self.current_block_index += 1
                self.current_block = BytesIO(self.load_block(self.current_block_index))
        return b"".join(rval)

    def __next__(self):
        line = self.readline()
        if line == b"":
            raise StopIteration

    def __iter__(self):
        return self

    def close(self):
        self.file.close()



"""
Utility functions for working with `Bitset`s and treating lists of (start,end)
as `Bitset`s.
"""

from bx.bitset import (
    BinnedBitSet,
    MAX,
)


def bitset_intersect(ex1, ex2):
    bits1 = list2bits(ex1)
    bits2 = list2bits(ex2)
    bits1.iand(bits2)
    return bits2list(bits1)


def bitset_subtract(ex1, ex2):
    bits1 = list2bits(ex1)
    bits2 = list2bits(ex2)
    bits2.invert()
    bits1.iand(bits2)
    return bits2list(bits1)


def list2bits(ex):
    bits = BinnedBitSet(MAX)
    for start, end in ex:
        bits.set_range(start, end - start)
    return bits


def bits2list(bits):
    ex = []
    end = 0
    while True:
        start = bits.next_set(end)
        if start == bits.size:
            break
        end = bits.next_clear(start)
        ex.append((start, end))
    return ex


def bitset_complement(exons):
    bits = BinnedBitSet(MAX)
    introns = []
    for start, end in exons:
        bits.set_range(start, end - start)
    bits.invert()

    # only complement within the range of the list
    ex_start = min([a[0] for a in exons])
    ex_end = max([a[1] for a in exons])
    end = ex_start
    len = ex_end
    while True:
        start = bits.next_set(end)
        if start == bits.size:
            break
        end = bits.next_clear(start)
        if end > len:
            end = len
        if start != end:
            introns.append((start, end))
        if end == len:
            break
    return introns


def bitset_interval_intersect(bits, istart, iend):
    rval = []
    end = istart
    len = iend
    while True:
        start = bits.next_set(end)
        if start >= len:
            break
        end = bits.next_clear(start)
        if start != end:
            rval.append((start, end))
        if end >= len:
            break
    return rval


def bitset_union(exons):
    bits = list2bits(exons)
    return bits2list(bits)



"""
Find clusters of intervals within a set of intervals.  A cluster is a
group (of size minregions) of intervals within a specific distance (of
mincols) of each other.

Returns Cluster objects, which have a chrom, start, end, and lines (a
list of linenumbers from the original file).  The original can then be
ran through with the linenumbers to extract clustered regions without
disturbing original order, or the clusters may themselves be written
as intervals.
"""

import math
import random

from bx.intervals.cluster import ClusterTree
from bx.intervals.io import GenomicInterval


def find_clusters(reader, mincols=1, minregions=2):
    extra = dict()
    chroms = dict()
    linenum = -1
    for interval in reader:
        linenum += 1
        if not isinstance(interval, GenomicInterval):
            extra[linenum] = interval
        else:
            if interval.chrom not in chroms:
                chroms[interval.chrom] = ClusterTree(mincols, minregions)
            try:
                chroms[interval.chrom].insert(interval.start, interval.end, linenum)
            except OverflowError as e:
                try:
                    # This will work only if reader is a NiceReaderWrapper
                    reader.skipped += 1
                    if reader.skipped < 10:
                        reader.skipped_lines.append((reader.linenum, reader.current_line, str(e)))
                except Exception:
                    pass
                continue
    return chroms, extra


# DEPRECATED: Use the ClusterTree in bx.intervals.cluster for this.
# It does the same thing, but is a C implementation.
class ClusterNode(object):
    def __init__(self, start, end, linenum, mincols, minregions):
        # Python lacks the binomial distribution, so we convert a
        # uniform into a binomial because it naturally scales with
        # tree size.  Also, python's uniform is perfect since the
        # upper limit is not inclusive, which gives us undefined here.
        self.priority = math.ceil((-1.0 / math.log(.5)) * math.log(-1.0 / (random.uniform(0, 1) - 1)))
        self.start = start
        self.end = end
        self.left = None
        self.right = None
        self.lines = [linenum]
        self.mincols = mincols
        self.minregions = minregions

    def insert(self, start, end, linenum):
        if start - self.mincols > self.end:
            # insert to right tree
            if self.right:
                self.right = self.right.insert(start, end, linenum)
            else:
                self.right = ClusterNode(start, end, linenum, self.mincols, self.minregions)
            # rebalance tree
            if self.priority < self.right.priority:
                return self.rotateleft()
        elif end + self.mincols < self.start:
            # insert to left tree
            if self.left:
                self.left = self.left.insert(start, end, linenum)
            else:
                self.left = ClusterNode(start, end, linenum, self.mincols, self.minregions)
            # rebalance tree
            if self.priority < self.left.priority:
                return self.rotateright()
        else:
            # insert here
            self.start = min(self.start, start)
            self.end = max(self.end, end)
            self.lines.append(linenum)
            # recursive call to push nodes up
            if self.left:
                self.left = self.left.push_up(self)
            if self.right:
                self.right = self.right.push_up(self)
        return self

    def rotateright(self):
        root = self.left
        self.left = self.left.right
        root.right = self
        return root

    def rotateleft(self):
        root = self.right
        self.right = self.right.left
        root.left = self
        return root

    def push_up(self, topnode):
        # Note: this function does not affect heap property
        # Distance method removed for inline, faster?
        distance = max(self.start, topnode.start) - min(self.end, topnode.end)
        if distance <= self.mincols:
            topnode.start = min(self.start, topnode.start)
            topnode.end = max(self.end, topnode.end)
            for linenum in self.lines:
                topnode.lines.append(linenum)
            if self.right:
                return self.right.push_up(topnode)
            if self.left:
                return self.left.push_up(topnode)
            return None
        if self.end < topnode.start and self.right:
            self.right = self.right.push_up(topnode)
        if self.start > topnode.end and self.left:
            self.left = self.left.push_up(topnode)
        return self

    def getintervals(self):
        if self.left:
            for start, end in self.left.getintervals(self.minregions):
                yield start, end
        if len(self.lines) >= self.minregions:
            yield self.start, self.end
        if self.right:
            for start, end in self.right.getintervals(self.minregions):
                yield start, end

    def getlines(self):
        if self.left:
            for line in self.left.getlines():
                yield line
        if len(self.lines) >= self.minregions:
            for line in self.lines:
                yield line
        if self.right:
            for line in self.right.getlines():
                yield line



"""
Various utilities.
"""

import bz2
import gzip


def open_compressed(filename, mode='r'):
    if filename.endswith(".bz2"):
        return bz2.BZ2File(filename, mode)
    elif filename.endswith(".gz"):
        return gzip.GzipFile(filename, mode)
    else:
        return open(filename, mode)



import random
from tempfile import mktemp

from . import interval_index_file
from .interval_index_file import Indexes


def test_offsets():
    assert interval_index_file.offsets_for_max_size(512*1024*1024 - 1) == [512 + 64 + 8 + 1, 64 + 8 + 1, 8 + 1, 1, 0]


def test_interval_index_file():
    ix = Indexes()
    chrs = []
    for i in range(5):
        intervals = []
        name = "seq%d" % i
        max = random.randint(0, interval_index_file.MAX)
        # print name, "size", max
        for i in range(500):
            start = random.randint(0, max)
            end = random.randint(0, max)
            if end < start:
                end, start = start, end
            ix.add(name, start, end, i, max=interval_index_file.MAX)
            intervals.append((start, end, i))
        chrs.append(intervals)
    fname = mktemp()
    f = open(fname, "wb")
    ix.write(f)
    f.close()
    del ix

    ix = Indexes(fname)
    for i in range(5):
        intervals = chrs[i]
        name = "seq%d" % i
        for i in range(100):
            start = random.randint(0, max)
            end = random.randint(0, max)
            if end < start:
                end, start = start, end
            query_intervals = set()
            for (s, e, i) in intervals:
                if e > start and s < end:
                    query_intervals.add((s, e, i))
            result = ix.find(name, start, end)
            for inter in result:
                assert inter in query_intervals


def test_zero():
    ix = Indexes()
    ix.add("t.idx", 0, 0, 1, 123)



#!/usr/bin/env python

"""
Randomly shuffle the columns of each block of a maf file. Note that this does
not change any other features of the maf block, thus the text of each row no
longer will match the sequence refered to by the other row attributes!

usage: %prog < maf > maf
"""

import sys

from bx import align


def __main__():

    maf_reader = align.maf.Reader(sys.stdin)
    maf_writer = align.maf.Writer(sys.stdout)

    for m in maf_reader:

        align.shuffle_columns(m)

        maf_writer.write(m)


if __name__ == "__main__":
    __main__()



#!/usr/bin/env python

"""
Find regions of first bed file that overlap regions in a second bed file. The
output preserves all fields from the input.

NOTE: -u and -d options are currently not functional!

usage: %prog bed_file_1 bed_file_2
    -m, --mincols=N: Require this much overlap (default 1bp)
    -u, --upstream_pad=N: upstream interval padding (default 0bp)
    -d, --downstream_pad=N: downstream interval padding (default 0bp)
    -v, --reverse: Print regions that DO NOT overlap
    -b, --booleans: Just print '1' if interval overlaps or '0' otherwise
"""
from __future__ import print_function

from warnings import warn

from bx.bitset_builders import binned_bitsets_from_file
from bx.cookbook import doc_optparse

mincols = 1
upstream_pad = 0
downstream_pad = 0

options, args = doc_optparse.parse(__doc__)
try:
    if options.mincols:
        mincols = int(options.mincols)
    if options.upstream_pad:
        upstream_pad = int(options.upstream_pad)
    if options.downstream_pad:
        downstream_pad = int(options.downstream_pad)
    reverse = bool(options.reverse)
    booleans = bool(options.booleans)
    in_fname, in2_fname = args
except Exception:
    doc_optparse.exit()

# Read first bed into some bitsets

bitsets = binned_bitsets_from_file(open(in2_fname))

# Read second BED and intersect

for line in open(in_fname):
    if line.startswith("#") or line.isspace():
        continue
    fields = line.split()
    start, end = int(fields[1]), int(fields[2])
    if start > end:
        warn("Bed interval start after end!")
    if fields[0] in bitsets and bitsets[fields[0]].count_range(start, end-start) >= mincols:
        if booleans:
            if reverse:
                print(0)
            else:
                print(1)
        elif not reverse:
            print(line, end=' ')
    else:
        if booleans:
            if reverse:
                print(1)
            else:
                print(0)
        elif reverse:
            print(line, end=' ')



#!/usr/bin/env python

"""
Read BED file and extend each record to the specified minimum length. If chromosome
size information is provided trim extended intervals.

usage: %prog amount [ chrom_file ] < bed_file
"""
from __future__ import print_function

import sys

from bx.intervals.io import GenomicIntervalReader

length = int(sys.argv[1])
chrom_len = None
if len(sys.argv) > 2:
    chrom_len = dict((fields[0], int(fields[1])) for fields in map(str.split, open(sys.argv[2])))

for interval in GenomicIntervalReader(sys.stdin):
    if interval.end - interval.start < length:
        start = interval.start
        end = interval.end
        # Extend in positive direction on strand
        if interval.strand == "+":
            end = start + length
        else:
            start = end - length
        # Trim
        if start < 0:
            start = 0
        if chrom_len and end > chrom_len[interval.chrom]:
            end = chrom_len[interval.chrom]
        # Set new start and end
        interval.start = start
        interval.end = end
    # Output possibly adjusted interval
    print(interval)



#!/usr/bin/env python

"""
Read a feature file containing a 0 or 1 on each line, output
all mafs whose index in maf_file corresponds to a row having a 1

usage: %prog feature_file < maf_file
"""

import sys

import bx.align.maf


def __main__():

    feature_file = sys.argv[1]

    if len(sys.argv) > 2:
        match = int(sys.argv[2])
    else:
        match = 1

    feature_vector = [int(line) for line in open(feature_file)]

    maf_reader = bx.align.maf.Reader(sys.stdin)
    maf_writer = bx.align.maf.Writer(sys.stdout)

    index = 0

    for m in maf_reader:
        if feature_vector[index] == match:
            maf_writer.write(m)
        index += 1


if __name__ == "__main__":
    __main__()
